<?php
/*
 * Copyright(c) 2015 PAYDESIGN, Inc. All rights reserved.
 * http://www.paydesign.co.jp/
 */

namespace Plugin\PayDesign\Controller;

use Eccube\Application;
use Eccube\Controller\AbstractController;
use Eccube\Common\Constant;
use Plugin\PayDesign\Controller\Util\PluginUtil;
use Plugin\PayDesign\Form\Type\MypageCreditType;
use Plugin\PayDesign\Form\Type\MypagePeriodicType;
use Plugin\PayDesign\Service\client\PAYDESIGN_PLG_DCHK_CREDIT;

/**
 * Controller to handle module setting screen
 */
class MypageController extends AbstractController {

    public $app;
    public $const;

    // クレジットカード情報表示・編集
    public function credit_index(Application $app)
    {
        // 初期化
        $this->app = $app;
        $this->const = $app['config']['PayDesign']['const'];
        $param = 0 ; 
        $msg = "" ;
        $err_message = "";
        $objClientCredit = new PAYDESIGN_PLG_DCHK_CREDIT($this->app);

        // プラグイン設定の取得
        $objMdl =& PluginUtil::getInstance($this->app);
        $objMdl->install();
        $userSettings = $objMdl->getUserSettings();

        // ログイン中の会員IDの取得
        $customer_id = $this->app['user']->getId();

        // 現在の通常注文用カード情報の取得
        $NormalCardInfo = $objClientCredit->getCardInfoDataForDisp($customer_id, $this->const['PDP_CARD_KIND_NORMAL']);

        // 現在の定期注文用カード情報の取得
        $PeriodicCardInfo = $objClientCredit->getCardInfoDataForDisp($customer_id, $this->const['PDP_CARD_KIND_PERIODIC']);

        $mode = $this->app['request']->get('mode');
        switch( $mode ) {
            case 'normal_card_info_change':
                $param = $this->const['PDP_CARD_KIND_NORMAL'] ;
                $msg = "クレジットリンク型決済（通常注文用カード情報更新）　遷移先URL：";
            case 'periodic_card_info_change':
                if( $param == 0 ) { $param = $this->const['PDP_CARD_KIND_PERIODIC'] ; }
                $redirect_url = $objClientCredit->getCardLinkUrl($param);
                if( $msg == "" ) {
                    $msg = "クレジットリンク型決済（定期文用カード情報更新）　遷移先URL："; }
                $msg .= $redirect_url;
                $objMdl->printLog($msg);
                if( $objMdl->checkEccubeOldVersion011() ) {
                    // 3.0.10以前のバージョン
                    header("Location: " . $redirect_url);
                    exit;
                } else {
                    // 3.0.11以降のバージョン
                    return $this->app->redirect($redirect_url);
                }
                break ;
            case 'normal_card_info_delete':
                $param = $this->const['PDP_CARD_KIND_NORMAL'] ;
                $card_kind_word = "通常注文用";
                $ip_user_id = $NormalCardInfo['card_sid'];
                $objMdl->printLog("クレジットリンク型決済（通常注文用カード情報削除）");
                if( $NormalCardInfo['card_num'] == "" ) {
                    $err_message = "【カード情報エラー】削除するカード情報が存在していません。";
                }
            case 'periodic_card_info_delete':
                if( $err_message == "" ) {
                    if( $param == 0 ) {
                        $param = $this->const['PDP_CARD_KIND_PERIODIC'] ;
                        $card_kind_word = "定期注文用";
                        $ip_user_id = $PeriodicCardInfo['card_sid'];
                        $objMdl->printLog("クレジットリンク型決済（定期注文用カード情報削除）");
                    }
                    $err_message = $objClientCredit->execCardInfoDelete($customer_id, $param);
                    if( $err_message == "" ) {
                        // セッションにデータを保存
                        $session = $this->app['request']->getSession();
                        $session->set('eccube.mypage.paydesign.credit_param', $param);
                        // 正常処理完了
                        $redirect_url = $this->app->url('paydesign_mypage_credit_delete_complete');
                        if( $objMdl->checkEccubeOldVersion011() ) {
                            // 3.0.10以前のバージョン
                            header("Location: " . $redirect_url);
                            exit;
                        } else {
                            // 3.0.11以降のバージョン
                            return $this->app->redirect($redirect_url);
                        }
                    }
                }
                break ;
            default:
                break ;
        }

        // マイページメニューの設定
        $BaseInfo = $this->app['eccube.repository.base_info']->get();
        $oldVersion = $objMdl->getEccubeOldVersion();
        $favorite = 0;
        if( in_array(Constant::VERSION, $oldVersion) ) {
            $version_flg = 0;
        } else {
            if( $BaseInfo->getOptionFavoriteProduct() == 1 ) { $favorite = 1; }
            $version_flg = 1;
        }
        // クレジット決済が有効か否か
        $credit_enable = 0;
        foreach( $userSettings['payment'] as $val ) {
            if( $val == "credit" ) { $credit_enable = 1; break; } }
        // ユーザーID決済が有効か否か
        $credit_uid_settle = 0;
        if( $userSettings["credit_uid_settle"] == "true" ) { $credit_uid_settle = 1; }
        // 定期機能が有効か否か
        $credit_periodic_buy = 0 ;
        if( $objMdl->getCreditPeriodicBuy() == 'true' ) { $credit_periodic_buy = 1; }

        // 対象会員に【継続中】、【停止】の定期情報があるかどうかを確認
        $periodic_enable_flg = 0;
        $qb = $this->app['eccube.plugin.paydesign.repository.paydesign_periodic_info']->getQueryBuilderBySearchEnablePeriodicData($customer_id);
        $Periodics = $qb->getQuery()->getResult();
        if( count($Periodics)>0 ) { $periodic_enable_flg = 1; }

        // フォームに渡すデータの作成
        $mypageCreditForm = new MypageCreditType($this->app);
        $form = $this->app['form.factory']->createBuilder($mypageCreditForm)->getForm();
        // 表示するフォームの設定
        return $app['view']->render('PayDesign/View/mypage_credit.twig', array(
             'favorite' => $favorite,
             'version_flg' => $version_flg,
             'form'  => $form->createView(),
             'normal_card_info' => $NormalCardInfo,
             'periodic_card_info' => $PeriodicCardInfo,
             'err_message' => $err_message,
             'card_kind' => $param,
             'credit_enable' => $credit_enable,
             'credit_uid_settle' => $credit_uid_settle,
             'credit_periodic_buy' => $credit_periodic_buy,
             'periodic_enable_flg' => $periodic_enable_flg,
        ));
    }

    // クレジットカード情報編集完了
    public function credit_complete(Application $app)
    {
        // 初期化
        $this->app = $app;
        $this->const = $app['config']['PayDesign']['const'];
        // 引数に表示するテンプレートファイル名を設定
        return $this->credit_complete_common('PayDesign/View/mypage_credit_complete.twig');
    }

    // クレジットカード情報削除完了
    public function credit_delete_complete(Application $app)
    {
        // 初期化
        $this->app = $app;
        $this->const = $app['config']['PayDesign']['const'];
        $objClientCredit = new PAYDESIGN_PLG_DCHK_CREDIT($this->app);

        // ログイン中の会員IDの取得
        $customer_id = $this->app['user']->getId();

        // プラグイン設定情報の取得
        $objMdl =& PluginUtil::getInstance($this->app);
        $objMdl->install();
        $userSettings = $objMdl->getUserSettings();

        // セッション情報の取得
        $param = 0;
        $card_kind_word = "";
        $session = $this->app['request']->getSession();
        if( $session->get('eccube.mypage.paydesign.credit_param') ) {
            $param = $session->get('eccube.mypage.paydesign.credit_param');
            if( $param == $this->const['PDP_CARD_KIND_NORMAL'] ) {
                $card_kind_word = "通常";
            } else {
                $card_kind_word = "定期";
            }
        }

        // IP_USER_IDの取得
        $ip_user_id = "";
        $PayDesignCardInfo = NULL;
        $PayDesignCardInfo = $objClientCredit->getCardInfoData("", $customer_id, $param);
        if( !is_null($PayDesignCardInfo) ) { $ip_user_id = $PayDesignCardInfo->getCardSid(); }

        // 正常完了時のメール送信
        $BaseInfo = $this->app['eccube.repository.base_info']->get();
        $subject = '[' . $BaseInfo->getShopName() . '] ' . $this->const['PDP_CREDIT_DELETE_SUBJECT'];
        $Customer = $this->app['eccube.repository.customer']->find($customer_id);
        $to_mailaddr = $Customer->getEmail();
        $formData = array(
            'name01'     => $Customer->getName01(),
            'name02'     => $Customer->getName02(),
            'shop_name'  => $BaseInfo->getShopName(),
            'card_kind'  => $card_kind_word,
            'ip_user_id' => $ip_user_id,
            'email'      => $BaseInfo->getEmail01(),
        );
        $objMdl->printLog("クレジットカード情報削除完了メール送信設定TO(".$to_mailaddr.")");
        $objMdl->printLog($formData);
        $body = $this->app->renderView("PayDesign/View/mail_template/delete_credit.twig", array(
            'data' => $formData,
        ));
        // メール送信実行
        $objMdl->sendMail( $subject, $to_mailaddr, $body );

        // (DBからの)カード情報削除処理
        $objClientCredit->execCardInfoDeleteFromDB($customer_id, $param);

        // 引数に表示するテンプレートファイル名を設定
        return $this->credit_complete_common('PayDesign/View/mypage_credit_delete_complete.twig');
    }

    // 定期情報一覧表示
    public function periodic_index(Application $app)
    {
        // 初期化
        $this->app = $app;
        $this->const = $app['config']['PayDesign']['const'];

        // プラグイン設定の取得
        $objMdl =& PluginUtil::getInstance($this->app);
        $objMdl->install();
        $userSettings = $objMdl->getUserSettings();

        // ログイン中の会員IDの取得
        $customer_id = $this->app['user']->getId();

        // 定期マスタ情報の取得(【継続中(1)】と【停止(2)】のみを表示)
        $PayDesignPeriodicInfo =
          $this->app['eccube.plugin.paydesign.repository.paydesign_periodic_info']->findBy(
              array('customer_id' => $customer_id,
                    'periodic_status' => array(1,2)
              ));
        if( !is_null($PayDesignPeriodicInfo) ) {
           $periodicCount = count($PayDesignPeriodicInfo);
           $periodic_info = $this->createPeriodicInfoAllForView($PayDesignPeriodicInfo);
        } else {
            $periodic_info = NULL;
            $periodicCount = 0;
        }

        // マイページメニューの設定
        $BaseInfo = $this->app['eccube.repository.base_info']->get();
        $oldVersion = $objMdl->getEccubeOldVersion();
        $favorite = 0;
        if( in_array(Constant::VERSION, $oldVersion) ) {
            $version_flg = 0;
        } else {
            if( $BaseInfo->getOptionFavoriteProduct() == 1 ) { $favorite = 1; }
            $version_flg = 1;
        }
        // クレジット決済が有効か否か
        $credit_enable = 0;
        foreach( $userSettings['payment'] as $val ) {
            if( $val == "credit" ) { $credit_enable = 1; break; } }
        // ユーザーID決済が有効か否か
        $credit_uid_settle = 0;
        if( $userSettings["credit_uid_settle"] == "true" ) { $credit_uid_settle = 1; }
        // 定期機能が有効か否か
        $credit_periodic_buy = 0 ;
        if( $objMdl->getCreditPeriodicBuy() == 'true' ) { $credit_periodic_buy = 1; }

        return $app['view']->render('PayDesign/View/mypage_periodic.twig', array(
             'favorite' => $favorite,
             'version_flg' => $version_flg,
             'credit_periodic_buy' => $credit_periodic_buy,
             'credit_enable'  => $credit_enable,
             'credit_uid_settle' => $credit_uid_settle,
             'periodic_count' => $periodicCount,
             'periodic_info' => $periodic_info,
        ));
    }

    // 定期情報の編集
    public function periodic_edit(Application $app, $id = null)
    {
        // 初期化
        $this->app = $app;
        $this->const = $app['config']['PayDesign']['const'];

        // プラグイン設定の取得
        $objMdl =& PluginUtil::getInstance($this->app);
        $objMdl->install();
        $userSettings = $objMdl->getUserSettings();

        // ログイン中の会員IDの取得
        $customer_id = $this->app['user']->getId();

        // 定期マスタ情報の取得
        $PayDesignPeriodicInfo = 
            $this->app['eccube.plugin.paydesign.repository.paydesign_periodic_info']->find($id);
        $periodic_info = NULL;
        $periodic_info = $this->createPeriodicInfoForView($PayDesignPeriodicInfo);

        // 定期商品情報の取得（対象商品が通常商品か初回登録のみ商品かの情報取得が目的）
        $product_class_id = $PayDesignPeriodicInfo->getProductClassId();
        $PayDesignProductPeriodicInfo =
            $this->app['eccube.plugin.paydesign.repository.paydesign_product_periodic_info']->find($product_class_id);
        $product_periodic_settle_method = $PayDesignProductPeriodicInfo->getPeriodicSettleMethod();

        // フォーム関連のデータ設定
        $form = $this->app['form.factory']
            ->createBuilder('paydesign_mypage_periodic')
            ->getForm();
        $form->get('periodic_status')->setData($PayDesignPeriodicInfo->getPeriodicStatus());
        $form->get('periodic_next_date')->setData($PayDesignPeriodicInfo->getPeriodicNextDate());
        $form->handleRequest($this->app['request']);

        // マイページメニューの設定
        $BaseInfo = $this->app['eccube.repository.base_info']->get();
        $oldVersion = $objMdl->getEccubeOldVersion();
        $favorite = 0;
        if( in_array(Constant::VERSION, $oldVersion) ) {
            $version_flg = 0;
        } else {
            if( $BaseInfo->getOptionFavoriteProduct() == 1 ) { $favorite = 1; }
            $version_flg = 1;
        }
        // クレジット決済が有効か否か
        $credit_enable = 0;
        foreach( $userSettings['payment'] as $val ) {
            if( $val == "credit" ) { $credit_enable = 1; break; } }
        // ユーザーID決済が有効か否か
        $credit_uid_settle = 0;
        if( $userSettings["credit_uid_settle"] == "true" ) { $credit_uid_settle = 1; }
        // 定期機能が有効か否か
        $credit_periodic_buy = 0 ;
        if( $objMdl->getCreditPeriodicBuy() == 'true' ) { $credit_periodic_buy = 1; }
        // 申込状況一覧を取得
        $periodicStatus = $objMdl->getPeriodicStatus();
        // 会員によるマイページからの次回注文日の変更を許可する
        $change_permission_periodic_next_date = 0;
        if( $objMdl->getChangePermissionPeriodicNextDate() == 'true' ) { $change_permission_periodic_next_date = 1; }
        // 会員によるマイページからの申込状況の変更を許可する
        $change_permission_periodic_status = 0;
        if( $objMdl->getChangePermissionPeriodicStatus() == 'true' ) { $change_permission_periodic_status = 1; }

        $err_message_pnd = "";
        $err_message_ps = "";
        $mode = $this->app['request']->get('mode');
        switch( $mode ) {
            case 'periodic_edit':
                if( $err_message_pnd == "" && $err_message_ps == "") {
                    if($form->isValid()) {
                        $now = new \DateTime();
                        $data = $form->getData();
                        if( $change_permission_periodic_next_date > 0 && $product_periodic_settle_method != 2 ) {
                            if( $now >= $data['periodic_next_date'] ) {
                                $err_message_pnd = "※ 次回注文日には過去の日付は指定する事ができません。";
                            }
                        }
                        if( $change_permission_periodic_status > 0 ) {
                            if( is_null($data['periodic_status']) || empty($data['periodic_status']) ) {
                                $err_message_ps = "※ 申込状況は必ず設定してください。";
                            }
                        }
                        if( $err_message_pnd == "" && $err_message_ps == "") {
                            // 定期情報の更新
                            if( $change_permission_periodic_next_date > 0 && $product_periodic_settle_method != 2 ) {
                                // 次回注文日
                                $PayDesignPeriodicInfo->setPeriodicNextDate($data['periodic_next_date']);
                                // 次回注文基準日(※次回注文日に合わせる)
                                $periodic_create_day = intval($data['periodic_next_date']->format('d'));
                                $PayDesignPeriodicInfo->setPeriodicCreateDay($periodic_create_day);
                            }
                            if( $change_permission_periodic_status > 0 ) {
                                $PayDesignPeriodicInfo->setPeriodicStatus($data['periodic_status']);
                            }
                            $PayDesignPeriodicInfo->setUpdateDate($now);
                            $this->app['orm.em']->persist($PayDesignPeriodicInfo);
                            $this->app['orm.em']->flush();

                            // セッションにデータを保存
                            $session = $this->app['request']->getSession();
                            $session->set('eccube.mypage.paydesign.periodicId', $periodic_info['id']);

                            // 正常処理完了
                            $redirect_url = $this->app->url('paydesign_mypage_periodic_complete');
                            if( $objMdl->checkEccubeOldVersion011() ) {
                                // 3.0.10以前のバージョン
                                header("Location: " . $redirect_url);
                                exit;
                            } else {
                                // 3.0.11以降のバージョン
                                return $this->app->redirect($redirect_url);
                            }
                        }
                    }
                }
                break ;
            default:
                break ;
        }

        return $app['view']->render('PayDesign/View/mypage_periodic_edit.twig', array(
             'form' => $form->createView(),
             'favorite' => $favorite,
             'version_flg' => $version_flg,
             'credit_periodic_buy' => $credit_periodic_buy,
             'change_permission_periodic_next_date' => $change_permission_periodic_next_date,
             'change_permission_periodic_status' => $change_permission_periodic_status,
             'credit_enable'  => $credit_enable,
             'credit_uid_settle' => $credit_uid_settle,
             'periodic_info' => $periodic_info,
             'product_periodic_settle_method' => $product_periodic_settle_method,
             'arrPeriodicStatus' => $periodicStatus,
             'err_message_pnd' => $err_message_pnd,
             'err_message_ps' => $err_message_ps,
        ));
    }

    // 定期情報の編集完了（完了画面の表示）
    public function periodic_complete(Application $app)
    {
        // 初期化
        $this->app = $app;
        $this->const = $app['config']['PayDesign']['const'];

        // プラグイン設定の取得
        $objMdl =& PluginUtil::getInstance($this->app);
        $objMdl->install();
        $userSettings = $objMdl->getUserSettings();
        $periodicStatus = $objMdl->getPeriodicStatus();

        // ログイン中の会員IDの取得
        $customer_id = $this->app['user']->getId();

        // マイページメニューの設定
        $BaseInfo = $this->app['eccube.repository.base_info']->get();
        $oldVersion = $objMdl->getEccubeOldVersion();
        $favorite = 0;
        if( in_array(Constant::VERSION, $oldVersion) ) {
            $version_flg = 0;
        } else {
            if( $BaseInfo->getOptionFavoriteProduct() == 1 ) { $favorite = 1; }
            $version_flg = 1;
        }
        // クレジット決済が有効か否か
        $credit_enable = 0;
        foreach( $userSettings['payment'] as $val ) {
            if( $val == "credit" ) { $credit_enable = 1; break; } }
        // ユーザーID決済が有効か否か
        $credit_uid_settle = 0;
        if( $userSettings["credit_uid_settle"] == "true" ) { $credit_uid_settle = 1; }
        // 定期機能が有効か否か
        $credit_periodic_buy = 0 ;
        if( $objMdl->getCreditPeriodicBuy() == 'true' ) { $credit_periodic_buy = 1; }

        // 定期情報IDの取得(取得出来た場合はセッション情報を削除)
        $periodic_id = 0;
        $session = $this->app['request']->getSession();
        if( $session->get('eccube.mypage.paydesign.periodicId') ) {
            $periodic_id = $session->get('eccube.mypage.paydesign.periodicId');
            $session->remove('eccube.mypage.paydesign.periodicId');
        }
        
        // 定期マスタ情報の取得
        if( $periodic_id > 0 ) {
            $PayDesignPeriodicInfo = 
                $this->app['eccube.plugin.paydesign.repository.paydesign_periodic_info']->find($periodic_id);
            if( !is_null($PayDesignPeriodicInfo) ) {
                // 完了メール送信
                $BaseInfo = $this->app['eccube.repository.base_info']->get();
                $subject = '[' . $BaseInfo->getShopName() . '] ' . $this->const['PDP_PERIODIC_UPDATE_SUBJECT'];
                $Customer = $this->app['eccube.repository.customer']->find($customer_id);
                $to_mailaddr = $Customer->getEmail();
                $formData = array(
                    'name01'             => $Customer->getName01(),
                    'name02'             => $Customer->getName02(),
                    'shop_name'          => $BaseInfo->getShopName(),
                    'email'              => $BaseInfo->getEmail01(),
                    'periodic_id'        => $periodic_id,
                    'periodic_next_date' => $PayDesignPeriodicInfo->getPeriodicNextDate()->format('Y/m/d'),
                    'periodic_status'    => $periodicStatus[$PayDesignPeriodicInfo->getPeriodicStatus()],
                );
                $body = $this->app->renderView("PayDesign/View/mail_template/update_periodic.twig", array(
                    'data' => $formData,
                ));
                // メール送信
                $objMdl->sendMail( $subject, $to_mailaddr, $body );
            }
        }

        return $app['view']->render('PayDesign/View/mypage_periodic_complete.twig', array(
             'favorite' => $favorite,
             'version_flg' => $version_flg,
             'credit_periodic_buy' => $credit_periodic_buy,
             'credit_enable'  => $credit_enable,
             'credit_uid_settle' => $credit_uid_settle,
        ));
    }

    // Myページ完了ページ表示処理（共通ロジック）
    private function credit_complete_common($filename)
    {
        // プラグイン設定の取得
        $objMdl =& PluginUtil::getInstance($this->app);
        $objMdl->install();
        $userSettings = $objMdl->getUserSettings();

        // マイページメニューの設定
        $BaseInfo = $this->app['eccube.repository.base_info']->get();
        $oldVersion = $objMdl->getEccubeOldVersion();
        $favorite = 0;
        if( in_array(Constant::VERSION, $oldVersion) ) {
            $version_flg = 0;
        } else {
            if( $BaseInfo->getOptionFavoriteProduct() == 1 ) { $favorite = 1; }
            $version_flg = 1;
        }
        $credit_periodic_buy = 0 ;
        if( $objMdl->getCreditPeriodicBuy() == 'true' ) { $credit_periodic_buy = 1; }

        // セッション情報の取得と破棄
        $card_kind = "";
        $session = $this->app['request']->getSession();
        if( $session->get('eccube.mypage.paydesign.credit_param') ) {
            $param = $session->get('eccube.mypage.paydesign.credit_param');
            $session->remove('eccube.mypage.paydesign.credit_param');
            if( $param == $this->const['PDP_CARD_KIND_NORMAL'] ) {
                $card_kind = "通常";
            } else {
                $card_kind = "定期";
            }
        }

        // 完了ページを表示
        return $this->app['view']->render($filename, array(
             'favorite' => $favorite,
             'version_flg' => $version_flg,
             'card_kind' => $card_kind,
             'credit_periodic_buy' => $credit_periodic_buy,
        ));
    }

    // 定期情報：表示用一覧データ作成処理
    private function createPeriodicInfoAllForView( $PayDesignPeriodicInfo )
    {
        $periodic_info = array();

        // プラグイン設定の取得
        $objMdl =& PluginUtil::getInstance($this->app);
        $objMdl->install();
        $periodicStatus = $objMdl->getPeriodicStatus();

        if( !is_null($PayDesignPeriodicInfo) ) {
            foreach( $PayDesignPeriodicInfo as $periodicInfo ) {

                // 商品情報の取得(商品画像の取得)
                $product_image = NULL;
                $Product = $this->app['eccube.repository.product']->find($periodicInfo->getProductId());
                if( !is_null($Product) ) {
                    $product_image = $Product->getMainListImage();
                }

                // 出力用の情報作成
                $buf = array();
                $buf["id"] = $periodicInfo->getId();
                $buf["periodic_status"] = $periodicStatus[$periodicInfo->getPeriodicStatus()];
                $buf["product_image"] = $product_image;
                $buf["product_name"] = $periodicInfo->getProductName();
                $buf["price"] = $periodicInfo->getPrice();
                $buf["quantity"] = $periodicInfo->getQuantity();
                $buf["periodic_next_date"] = $periodicInfo->getPeriodicNextDate();
                $buf["periodic_interval"] = $periodicInfo->getPeriodicInterval();
                $periodic_info[] = $buf;
            }
        }

        return $periodic_info;
    }

    private function createPeriodicInfoForView( $PayDesignPeriodicInfo )
    {
        $periodic_info = array();

        // プラグイン設定の取得
        $objMdl =& PluginUtil::getInstance($this->app);
        $objMdl->install();
        $periodicStatus = $objMdl->getPeriodicStatus();

        if( !is_null($PayDesignPeriodicInfo) ) {

            // 商品情報の取得(商品画像の取得)
            $product_image = NULL;
            $Product = $this->app['eccube.repository.product']->find($PayDesignPeriodicInfo->getProductId());
            if( !is_null($Product) ) {
                $product_image = $Product->getMainListImage();
            }

            // 都道府県の取得
            $pref = "";
            $Pref = $this->app['eccube.repository.master.pref']->find($PayDesignPeriodicInfo->getShippingPref());
            $pref = $Pref->getName();

            $periodic_info["id"] = $PayDesignPeriodicInfo->getId();
            $periodic_info["periodic_next_date"] = $PayDesignPeriodicInfo->getPeriodicNextDate();
            $periodic_info["periodic_status"] = $periodicStatus[$PayDesignPeriodicInfo->getPeriodicStatus()];
            $periodic_info["periodic_status_id"] = $PayDesignPeriodicInfo->getPeriodicStatus();
            $periodic_info["periodic_interval"] = $PayDesignPeriodicInfo->getPeriodicInterval();
            $periodic_info["product_image"] = $product_image;
            $periodic_info["product_id"] = $PayDesignPeriodicInfo->getProductId();
            $periodic_info["product_name"] = $PayDesignPeriodicInfo->getProductName();
            $periodic_info["price"] = $PayDesignPeriodicInfo->getPrice();
            $periodic_info["quantity"] = $PayDesignPeriodicInfo->getQuantity();
            $periodic_info["total_price"] = $periodic_info["price"] * $periodic_info["quantity"];
            $periodic_info["payment_method"] = $PayDesignPeriodicInfo->getPaymentMethod();
            $periodic_info["shipping_name01"] = $PayDesignPeriodicInfo->getShippingName01();
            $periodic_info["shipping_name02"] = $PayDesignPeriodicInfo->getShippingName02();
            $periodic_info["shipping_kana01"] = $PayDesignPeriodicInfo->getShippingKana01();
            $periodic_info["shipping_kana02"] = $PayDesignPeriodicInfo->getShippingKana02();
            $periodic_info["shipping_zip01"] = $PayDesignPeriodicInfo->getShippingZip01();
            $periodic_info["shipping_zip02"] = $PayDesignPeriodicInfo->getShippingZip02();
            $periodic_info["shipping_pref"] = $pref;
            $periodic_info["shipping_addr01"] = $PayDesignPeriodicInfo->getShippingAddr01();
            $periodic_info["shipping_addr02"] = $PayDesignPeriodicInfo->getShippingAddr02();
            $periodic_info["shipping_tel01"] = $PayDesignPeriodicInfo->getShippingTel01();
            $periodic_info["shipping_tel02"] = $PayDesignPeriodicInfo->getShippingTel02();
            $periodic_info["shipping_tel03"] = $PayDesignPeriodicInfo->getShippingTel03();
        }

        return $periodic_info;
    }
}
?>
