<?php
/*
 * Copyright(c) 2015 PAYDESIGN, Inc. All rights reserved.
 * http://www.paydesign.co.jp/
 */

namespace Plugin\PayDesign;

use Eccube\Application;
use Eccube\Common\Constant;
use Eccube\Event\RenderEvent;
use Eccube\Event\ShoppingEvent;
use Plugin\PayDesign\Controller\Util\PluginUtil;
use Plugin\PayDesign\Controller\Util\CommonUtil;
use Plugin\PayDesign\Form\Type\AdminProductPeriodicInfoType;
use Plugin\PayDesign\Service\client\PAYDESIGN_PLG_DCHK_CREDIT;
use Plugin\PayDesign\Service\client\PAYDESIGN_PLG_DCHK_CONVENIENCE;
use Plugin\PayDesign\Service\client\PAYDESIGN_PLG_DCHK_PERIODIC;
use Symfony\Component\Form\FormError;
use Symfony\Component\HttpKernel\Event\FilterResponseEvent;
use Symfony\Component\CssSelector\CssSelector;
use Symfony\Component\DomCrawler\Crawler;
use Symfony\Component\Validator\Constraints as Assert;

class PayDesign {

    private $app;
    private $CreditUidCardSelectFlg = array(0) ;
    private $ConfirmErrMessage = "" ;

    public function __construct($app)
    {
        $this->app = $app;
    }

    // イベントハンドラーで呼ばれたイベント処理の記述

    // 【フロントページ】マイページALL：Render/Before
    public function onRenderMypageBefore(FilterResponseEvent $event) {

        $BaseInfo = $this->app['eccube.repository.base_info']->get();
        $objMdl =& PluginUtil::getInstance($this->app);
        $objMdl->install();
        $userSettings = $objMdl->getUserSettings();
        $oldVersion = $objMdl->getEccubeOldVersion();

        // マイページ ナビ選択肢の設定(URLより判断)
        // デフォルトのマイページメニューが増えた場合、修正の必要性あり。
        $param = array( 'mypageno' => 'index' ) ;
        $url = $event->getRequest()->getPathInfo();
        if( strstr($url, 'change') ) { $param = array( 'mypageno' => 'change' ) ; }
        if( strstr($url, 'delivery') ) { $param = array( 'mypageno' => 'delivery' ) ; }
        if( strstr($url, 'withdraw') ) { $param = array( 'mypageno' => 'withdraw' ) ; }
        // 以下2つは追加した機能
        if( strstr($url, 'credit') ) { $param = array( 'mypageno' => 'credit' ) ; }
        if( strstr($url, 'periodic') ) { $param = array( 'mypageno' => 'periodic' ) ; }
        // 以下の文字を含んだURLの場合は全てのナビメニューを選択可能とする。
        if( strstr($url, 'favorite') ) { $param = array() ; }
        if( strstr($url, 'login') ) { $param = array() ; }
        if( strstr($url, 'mail') ) { $param = array() ; }

        // マイページの表示メニューカスタマイズ
        $param['favorite'] = 0;
        if( in_array(Constant::VERSION, $oldVersion) ) {
            $class_name = 'local_nav';
            $param['version_flg'] = 0;
        } else { 
            if( $BaseInfo->getOptionFavoriteProduct() == 1 ) {
                $class_name = 'local_nav favorite';
                $param['favorite'] = 1;
            } else {
                $class_name = 'local_nav ';
            }
            $param['version_flg'] = 1;
        }
        // クレジット決済の有無の確認(クレジット決済が有効の場合のみクレジットカード情報編集画面を表示)
        $param['credit_enable'] = 0;
        foreach( $userSettings['payment'] as $val ) {
            if( $val == "credit" ) { $param['credit_enable'] = 1; break; } }
        // ユーザーID決済が有効か否か
        $param['credit_uid_settle'] = 0;
        if( $userSettings["credit_uid_settle"] == 'true' ) { $param['credit_uid_settle'] = 1; }
        // 定期機能が有効か否か
        $param['credit_periodic_buy'] = 0;
        if( $objMdl->getCreditPeriodicBuy() == 'true' ) { $param['credit_periodic_buy'] = 1; }
        $append_param = $param;
        $twig_filename = 'PayDesign/View/mypage_local_navi.twig';
        $this->changeHtmlView($event, $class_name, $twig_filename, $append_param);
    }

    // 【フロントページ】マイページ退会手続き：Controller/Before
    public function onControllerMypageWithdrawBefore() {

        // [memo]
        // 退会手続きをするとメールアドレスにダミーがセットと退会フラグをセットされる

        // ログイン中の会員ID
        $customer_id = $this->app['user']->getId();

        // カード情報と定期マスタの削除
        $this->deleteCardInfoAndPeriodicInfo($customer_id);
    }

    // 【フロントページ】商品詳細：Render/Before
    public function onRenderProductDetailBefore(FilterResponseEvent $event) {
    }

    // 【フロントページ】注文情報入力：Render/Before
    public function onRenderShoppingBefore(FilterResponseEvent $event) {

        $objMdl =& PluginUtil::getInstance($this->app);
        $objMdl->install();
        $userSettings = $objMdl->getUserSettings();

        // 注文情報の取得
        $Order = NULL;
        if( !is_null($this->app['eccube.service.cart']->getPreOrderId()) ) {
            $Order = $this->app['eccube.repository.order']->findOneBy(array('pre_order_id' => $this->app['eccube.service.cart']->getPreOrderId()));
        } else {
            // pre_order_id が無い場合、セッション情報に保存したorder_idからの取得を試みる
            $session = $this->app['request']->getSession();
            if( !is_null($session->get('eccube.plugin.paydesign.orderId')) ) {
                $Order = $this->app['eccube.repository.order']->find($session->get('eccube.plugin.paydesign.orderId'));
            }
        }

        if (!is_null($Order)) {

            // 注文完了後のブラウザバックによるエラー対策
            // 対象注文に order_date が存在している場合にこの処理に来た場合はエラーとする。
            $order_date = $Order->getOrderDate();
            if( !is_null($order_date) && !empty($order_date) ) {
                // エラー発生時はエラー画面に遷移する
                return $this->app->redirect($this->app->url('shopping_error'));
            }

            // 受注データを取得(最初に取得してる$Orderオブジェクトを使用)
            $oldVersion = $objMdl->getEccubeOldVersion();
            if( in_array(Constant::VERSION, $oldVersion) ) {
                $form = $this->app['form.factory']->createBuilder('shopping')->getForm();
                $cartService = $this->app['eccube.service.cart'];
                $orderService = $this->app['eccube.service.order'];

                $shippings = $Order->getShippings();
                $deliveries = $orderService->findDeliveriesFromOrderDetails($this->app, $Order->getOrderDetails());
                $delivery = $shippings[0]->getDelivery();

                $orderService->setFormDelivery($form, $deliveries, $delivery);       // 配送業者の設定
                $orderService->setFormDeliveryDate($form, $Order, $this->app);       // お届け日の設定
                $orderService->setFormDeliveryTime($form, $delivery);                // お届け時間の設定
                $orderService->setFormPayment($form, $delivery, $Order, $this->app); // 支払い方法選択
            } else {
                $form = $this->app['eccube.service.shopping']->getShippingForm($Order);
            }

            $Payment = $Order->getPayment();
            $PaymentConfig = null;
            if (!is_null($Payment)) {
                $PaymentConfig = $this->app['eccube.plugin.paydesign.repository.paydesign_payment_method']->find($Payment->getId());
            }

            // クレジット決済の支払方法名称の取得
            $credit_payment_method = $objMdl->getPaymentCreditMethod();
            // ユーザID決済フラグのフォームを設定
            $this->setCreditUidCardSelectFlg($form);

            // 定期商品が含まれているかどうか
            $periodic_flg = false;
            $objClientPeriodic = new PAYDESIGN_PLG_DCHK_PERIODIC($this->app);
            $periodic_flg = $objClientPeriodic->checkPeriodicProducts($Order);
            // 注文商品構成による購入パターンの取得
            $product_pattern = $objClientPeriodic->getProductPattern($Order);
            // dump($product_pattern);
            // dump("です");
            // 登録のみ商品だけの場合は各金額を\0に設定する（手数料はこの対象外）
            if( $product_pattern == 2 ) {
                $Order->setDeliveryFeeTotal(0);
                $Order->setSubTotal(0);
              //$Order->setCharge(0);
                $Order->setTax(0);
                $Order->setTotal(0);
                $Order->setPaymentTotal(0);
                $shippings = $Order->getShippings();
                foreach ($shippings as $Shipping) {
                    $Shipping->setShippingDeliveryFee(0);
                }
                $this->app['orm.em']->persist($Order);
                $this->app['orm.em']->flush();
            }

            $non_member_flg = false;
            $card_info_flg = false;
            if( is_null($Order->getCustomer()) ) {
                // 非会員フラグの設定
                $non_member_flg = true;
                if( $periodic_flg ) {
                    // 非会員且つ定期商品が含まれている場合のエラーメッセージ
                    $this->ConfirmErrMessage = "非会員の方は定期商品の購入をする事をは出来ません。"; }
            } else {
                // クレジットカード決済用のカード情報取得
                $arrRet = array();
                $customer_id = $Order->getCustomer()->getId();
                $objClientCredit = new PAYDESIGN_PLG_DCHK_CREDIT($this->app);
                // 定期商品が含まれていない場合のカード情報取得対象は通常クレジットカード
                $card_kind = $this->app['config']['PayDesign']['const']['PDP_CARD_KIND_NORMAL'];
                if( $periodic_flg ) {
                    // 定期商品が含まれている場合のカード情報取得対象は定期用クレジットカード
                    $card_kind = $this->app['config']['PayDesign']['const']['PDP_CARD_KIND_PERIODIC'];
                }
                $arrRet = $objClientCredit->getCardInfoDataForDisp($customer_id, $card_kind);
                // カード番号が存在し、有効であればフラグをセット
                if( $arrRet["card_num"] != "" && $arrRet["card_enable"] == $this->app['config']['PayDesign']['const']['PDP_ENABLE'] ) { $card_info_flg = true; }
            }

            // ユーザーID決済が有効かどうか
            $use_uid_flg = false;
            if( $userSettings["credit_uid_settle"] == "true" ) { $use_uid_flg = true; }

            // 支払い方法一覧を変更(上記の受注データが必要)
            $class_name = 'payment_list';
            $twig_filename = 'PayDesign/View/shopping_payment_list.twig';
            $append_param = array(
                'form' => $form->createView(),
                'credit_payment_method' => $credit_payment_method,
                'periodic_flg' => $periodic_flg,
                'non_member_flg' => $non_member_flg,
                'card_info_flg' => $card_info_flg,
                'use_uid_flg' => $use_uid_flg,
                'confirm_err_message' => $this->ConfirmErrMessage,
            );
            $this->changeHtmlView($event, $class_name, $twig_filename, $append_param);

            if (!is_null($PaymentConfig)) {
                if( $periodic_flg ) {
                    // 支払情報の変更（登録のみ商品\0表示対策）
                    $twig_filename = 'PayDesign/View/shopping_payment_info.twig';
                    $class_name = 'total_box';
                    $append_param = array(
                        'Order' => $Order,
                        'periodic_flg' => $periodic_flg,
                        'non_member_flg' => $non_member_flg,
                    );
                    $this->changeHtmlView($event, $class_name, $twig_filename, $append_param);
                } else {
                    // 購入ボタンの表示を変更
                    if( $periodic_flg ) {
                        // 定期商品の場合は【クレジットカード決済】固定
                        $payment_method = $objMdl->getPaymentCreditMethod();
                    } else {
                        $payment_method = $PaymentConfig->getMethod() ;
                    }
                    if( in_array(Constant::VERSION, $oldVersion) ) {
                        $class_name = 'btn btn-primary btn-block'; // old Ver.
                    } else {
                        $class_name = 'btn btn-primary btn-block prevention-btn prevention-mask'; // new Ver.
                    }
                    $twig_filename = 'PayDesign/View/next_cart_button.twig';
                    $append_param = array(
                        'button_comment' => $payment_method . 'へ',
                        'periodic_flg' => $periodic_flg,
                        'non_member_flg' => $non_member_flg,
                    );
                    $this->changeHtmlView($event, $class_name, $twig_filename, $append_param);
                }
            }

            // 処理終了後、エラーメッセージを初期化する
            $this->ConfirmErrMessage = "" ;
        } else {
            // 注文情報取得失敗時はエラー画面に遷移する
          //$this->app->addError('front.shopping.order.error');
            return $this->app->redirect($this->app->url('shopping_error'));
        }
    }

    public function onRenderShoppingPaymentBefore(FilterResponseEvent $event) {
        $this->CreditUidCardSelectFlg = array(1) ;
        $this->onRenderShoppingBefore($event);
    }

    public function onControllerShoppingPaymentBefore() {
        $objMdl =& PluginUtil::getInstance($this->app);
        $objMdl->install();
        $oldVersion = $objMdl->getEccubeOldVersion();

        $cartService = $this->app['eccube.service.cart'];
        $orderService = $this->app['eccube.service.order'];
        $orderRepository = $this->app['eccube.repository.order'];

        // 注文情報の取得
        $Order = NULL;
        if( !is_null($this->app['eccube.service.cart']->getPreOrderId()) ) {
            $Order = $this->app['eccube.repository.order']->findOneBy(array('pre_order_id' => $this->app['eccube.service.cart']->getPreOrderId()));
        } else {
            // pre_order_id が無い場合、セッション情報に保存したorder_idからの取得を試みる
            $session = $this->app['request']->getSession();
            if( !is_null($session->get('eccube.plugin.paydesign.orderId')) ) {
                $Order = $this->app['eccube.repository.order']->find($session->get('eccube.plugin.paydesign.orderId'));
            }
        }

        if( !is_null($Order) ) {

            // 注文完了後のブラウザバックによるエラー対策
            // 対象注文に order_date が存在している場合にこの処理に来た場合はエラーとする。
            $order_date = $Order->getOrderDate();
            if( !is_null($order_date) && !empty($order_date) ) {
                // エラー発生時はエラー画面に遷移する
                return $this->app->redirect($this->app->url('shopping_error'));
            }

            if( in_array(Constant::VERSION, $oldVersion) ) {
                $form = $this->app['form.factory']->createBuilder('shopping')->getForm();

                $shippings = $Order->getShippings();
                $deliveries = $orderService->findDeliveriesFromOrderDetails($this->app, $Order->getOrderDetails());
                $delivery = $shippings[0]->getDelivery();

                $orderService->setFormDelivery($form, $deliveries, $delivery);       // 配送業社の設定
                $orderService->setFormDeliveryDate($form, $Order, $this->app);       // お届け日の設定
                $orderService->setFormDeliveryTime($form, $delivery);                // お届け時間の設定
                $orderService->setFormPayment($form, $delivery, $Order, $this->app); // 支払い方法選択
            } else {
                $form = $this->app['eccube.service.shopping']->getShippingForm($Order);
            }

            // 追加したフォームの再設定
            $this->setCreditUidCardSelectFlg($form);

            if ('POST' === $this->app['request']->getMethod()) {

                $form->handleRequest($this->app['request']);

                if ($form->isValid()) {
                    $data = $form->getData();
                    $payment = $data['payment'];

                    $Order->setPayment($payment);
                    $Order->setPaymentMethod($payment->getMethod());
                    $Order->setCharge($payment->getCharge());

                    $total = $Order->getSubTotal() + $Order->getCharge() + $Order->getDeliveryFeeTotal();

                    $Order->setTotal($total);
                    $Order->setPaymentTotal($total);

                    // 受注関連情報を最新状態に更新
                    $this->app['orm.em']->flush();
                }
            }
            return $this->app->redirect($this->app->url('shopping'));
        } else {
            // 注文情報取得失敗時はエラー画面に遷移する
          //$this->app->addError('front.shopping.order.error');
            return $this->app->redirect($this->app->url('shopping_error'));
        }

    }

    // 【フロントページ】注文確認：Render/Before
    public function onRenderShoppingConfirmBefore(FilterResponseEvent $event) {
        // 注文確認でRender/Before処理を行う場合は注文情報入力のRender/Before処理と同じ
        $this->CreditUidCardSelectFlg = array(1) ;
        $this->onRenderShoppingBefore($event);
    }

    // 【フロントページ】注文確認：Controller/Before
    public function onControllerShoppingConfirmBefore($event = NULL) {
        $objMdl =& PluginUtil::getInstance($this->app);
        $objMdl->install();

        // 注文情報の取得
        $Order = NULL;
        if( !is_null($this->app['eccube.service.cart']->getPreOrderId()) ) {
            $Order = $this->app['eccube.repository.order']->findOneBy(array('pre_order_id' => $this->app['eccube.service.cart']->getPreOrderId()));
        } else {
            // pre_order_id が無い場合、セッション情報に保存したorder_idからの取得を試みる
            $session = $this->app['request']->getSession();
            if( !is_null($session->get('eccube.plugin.paydesign.orderId')) ) {
                $Order = $this->app['eccube.repository.order']->find($session->get('eccube.plugin.paydesign.orderId'));
            }
        }

        if( !is_null($Order) ) {

            // 注文完了後のブラウザバックによるエラー対策
            // 対象注文に order_date が存在している場合にこの処理に来た場合はエラーとする。
            $order_date = $Order->getOrderDate();
            if( !is_null($order_date) && !empty($order_date) ) {
                // エラー発生時はエラー画面に遷移する
                return $this->app->redirect($this->app->url('shopping_error'));
            }

            $oldVersion = $objMdl->getEccubeOldVersion();
            if( in_array(Constant::VERSION, $oldVersion) ) {
                $form = $this->app['form.factory']->createBuilder('shopping')->getForm();
                $deliveries = $this->findDeliveriesFromOrderDetails($this->app, $Order->getOrderDetails());

                // 配送業社の設定
                $shippings = $Order->getShippings();
                $delivery = $shippings[0]->getDelivery();

                // Formのカスタマイズ
                $this->setFormDelivery($form, $deliveries, $delivery);           // 配送業社の設定
                $this->setFormDeliveryDate($form, $Order, $this->app);           // お届け日の設定
                $this->setFormDeliveryTime($form, $delivery);                    // お届け時間の設定
                $this->setFormPayment($form, $delivery, $Order->getPayment());   // 支払い方法選択
            } else {
                $form = $this->app['eccube.service.shopping']->getShippingForm($Order);
            }
            $this->setCreditUidCardSelectFlg($form);                             // ユーザID決済の設定

            if ('POST' === $this->app['request']->getMethod()) {

                $form->handleRequest($this->app['request']);

                if ($form->isValid()) {

                    $formData = $form->getData();
                    $PayDesignPaymentMethod = $this->app['eccube.plugin.paydesign.repository.paydesign_payment_method']->find($formData['payment']->getId());

                    if (!is_null($PayDesignPaymentMethod)) {
//                        $this->app['session']->set('paydesign_formData',$formData);
                        $this->app['session']->set('paydesign_formData',json_encode($formData));

                        // 2016.2.13 注文ステータスは変更せず、購入処理中のままとする
                        // $Order->setOrderStatus($this->app['eccube.repository.order_status']->find($this->app['config']['order_pending']));
                        // 2016.11.17 受注情報、配送情報、メッセージ等を更新
                        $this->app['eccube.service.order']->setOrderUpdate($this->app['orm.em'], $Order, $formData);
                        // 2017.1.4 Ver.3.0.5からの setOrderUpdate の仕様変更を考慮【購入処理中】を維持する
                        $Order->setOrderStatus($this->app['eccube.repository.order_status']->find($this->app['config']['order_processing']));
                        // 2017.1.4 同時に注文日をこの時点で設定されてはいけないので初期化する
                        $Order->setOrderDate(NULL);
                        $this->app['orm.em']->persist($Order);
                        $this->app['orm.em']->flush();

                        // クレジット決済の支払方法名称の取得
                        $userSettings = $objMdl->getUserSettings();
                        $credit_payment_method = $objMdl->getPaymentCreditMethod();

                        // credit_uid_card_select_flg パラメータの取得
                        $credit_uid_card_select_flg = 0 ;
                        if( count($form->get('credit_uid_card_select_flg')->getData()) > 0 ) {
                            $credit_uid_card_select_flg = 1 ; }

                        /*********************************************************************
                         *
                         * 2017.1 EC-CUBEのバージョンアップにより、このタイミングでの作成はNGとなった。
                         *        従って、ここではコメントアウト（未使用）とする。
                         *
                        // 拡張受注情報を存在していなければ作成
                        $PaydesignOrderInfo =
                            $this->app['eccube.plugin.paydesign.repository.paydesign_order_info']->findOneBy( array( 'order_id' => $Order->getId(), 'order_kind' => $this->app['config']['PayDesign']['const']['PDP_ORDER_KIND_NORMAL']));
                        if( is_null($PaydesignOrderInfo) ) {
                            $PaydesignOrderInfo = new \Plugin\PayDesign\Entity\PayDesignOrderInfo() ; }
                        // データの設定
                        $PaydesignOrderInfo
                            ->setOrderId($Order->getId())
                            ->setSid($userSettings['prefix_word'].$Order->getId())
                            ->setOrderKind($this->app['config']['PayDesign']['const']['PDP_ORDER_KIND_NORMAL'])
                            ->setCreditUidCardSelectFlg($credit_uid_card_select_flg)
                            ->setCheckCount(0);
                        // 登録・更新処理実行
                        $this->app['orm.em']->persist($PaydesignOrderInfo);
                        $this->app['orm.em']->flush();
                        *********************************************************************/

                        // セッション情報に注文IDを保存
                        $this->app['session']->set('eccube.plugin.paydesign.orderId', $Order->getId());

                        // 決済方法が増えた場合は分岐が増えるので注意
                        if( $PayDesignPaymentMethod->getMethod() == $credit_payment_method ) {
                            // クレジット決済の場合
                            $objClientCredit = new PAYDESIGN_PLG_DCHK_CREDIT($this->app);
                            // 定期商品が含まれているかどうか
                            $periodic_flg = false;
                            $periodic_flg = $objClientCredit->checkPeriodicProducts($Order);
                            // 決済パターンの取得
                            $creditSettlementPattern = 0;
                            $creditSettlementPattern = $objClientCredit->getCreditSettlementPattern($Order, $credit_uid_card_select_flg);
                            $objMdl->printLog("決済パターン：".$creditSettlementPattern);
                            $objMdl->printLog("登録済カード使用：".$credit_uid_card_select_flg);
                            
                            if( $creditSettlementPattern != 1 &&
                                $creditSettlementPattern != 3 && $credit_uid_card_select_flg > 0 ) {
                                // 【前回の買い物で使用したカード情報を利用する】を使用する場合
                                if( is_null( $Order->getCustomer() ) ) {
                                    // 非会員の場合は機能を使用出来ないのでエラー
                                    $this->ConfirmErrMessage =
                                        "非会員の方は本機能を利用する事が出来ません。別の決済方法をご利用ください。" ;
                                    // 元のステータス(購入処理中)に戻す
                                    $Order->setOrderStatus($this->app['eccube.repository.order_status']->find($this->app['config']['order_processing']));
                                    $this->app['orm.em']->persist($Order);
                                    $this->app['orm.em']->flush();
                                    // エラー発生時は保存したセッション情報を削除
                                    $this->app['session']->remove('eccube.plugin.paydesign.orderId');
                                } else {
                                    $customer_id = $Order->getCustomer()->getId();
                                    // 定期商品が含まれていない場合のカード情報取得対象は通常クレジットカード
                                    $card_kind = $this->app['config']['PayDesign']['const']['PDP_CARD_KIND_NORMAL'];
                                    if( $periodic_flg ) {
                                        // 定期商品が含まれている場合のカード情報取得対象は定期用クレジットカード
                                        $card_kind = $this->app['config']['PayDesign']['const']['PDP_CARD_KIND_PERIODIC'];
                                    }
                                    $PaydesignCardInfo =
                                        $this->app['eccube.plugin.paydesign.repository.paydesign_card_info']->findOneBy(
                                            array( 'customer_id' => $customer_id,
                                                   'card_kind'   => $card_kind
                                        ));
                                    if( is_null($PaydesignCardInfo) ) {
                                        // 登録済みカード情報がなければエラー
                                        $this->ConfirmErrMessage =
                                            "登録済みカード情報が存在しません。カード情報をご確認ください。" ;
                                        // 元のステータス(購入処理中)に戻す
                                        $Order->setOrderStatus($this->app['eccube.repository.order_status']->find($this->app['config']['order_processing']));
                                        $this->app['orm.em']->persist($Order);
                                        $this->app['orm.em']->flush();
                                        // エラー発生時は保存したセッション情報を削除
                                        $this->app['session']->remove('eccube.plugin.paydesign.orderId');
                                    } else {
                                        // 購入処理実行
                                        $this->ConfirmErrMessage = $objClientCredit->execSettlement($Order, $PaydesignCardInfo);
                                        if($this->ConfirmErrMessage == "" ) {
                                            // 正常購入完了（メール送信ページに遷移）
                                            $redirect_url = $this->app->url('paydesign_shopping_credit_complete');
                                          //if( $event ) {
                                            if( !$objMdl->checkEccubeOldVersion011() ) {
                                                // 3.0.11以降
                                                $response = $this->app->redirect($redirect_url);
                                                $event->setResponse($response);
                                                return;
                                            } else {
                                                // 3.0.10以前
                                                header("Location: " . $redirect_url);
                                                exit;
                                            }
                                        } else {
                                            // エラー発生時は保存したセッション情報を削除
                                            $this->app['session']->remove('eccube.plugin.paydesign.orderId');
                                        }
                                    }
                                }
                            } else {
                                
                                // リンク型決済を使用する場合
                                // dump($creditSettlementPattern);
                                // exit();
                                // 2017.1 拡張受注情報の作成
                                $PaydesignOrderInfo = $objClientCredit->initPaydesignOrderInfo( $Order, $this->app['config']['PayDesign']['const']['PDP_ORDER_KIND_NORMAL'], '', $credit_uid_card_select_flg, 0 );
                                switch($creditSettlementPattern) {
                                    case  1:
                                        // $creditSettlementPattern = 1 の場合はカード登録のみ
                                        $redirect_url = $objClientCredit->getCardLinkOrderUrl($Order);
                                        $msg = "クレジットリンク型決済(カード登録のみ)　遷移先URL：\n".$redirect_url;
                                        $objMdl->printLog($msg);
                                      //if( $event ) {
                                        if( !$objMdl->checkEccubeOldVersion011() ) {
                                            // 3.0.11以降
                                            $response = $this->app->redirect($redirect_url);
                                            $event->setResponse($response);
                                            return;
                                        } else {
                                            // 3.0.10以前
                                            header("Location: " . $redirect_url);
                                            exit;
                                        }
                                        break;
                                    case  3:
                                        // $creditSettlementPattern = 3 の場合は決済側との処理をする必要はなし
                                        $objClientCredit->execNoSettlement($Order);
                                        $redirect_url = $this->app->url('paydesign_shopping_credit_complete');
                                      //if( $event ) {
                                        if( !$objMdl->checkEccubeOldVersion011() ) {
                                            // 3.0.11以降
                                            $response = $this->app->redirect($redirect_url);
                                            $event->setResponse($response);
                                            return;
                                        } else {
                                            // 3.0.10以前
                                            header("Location: " . $redirect_url);
                                            exit;
                                        }
                                        break;
                                    default:
                                        $redirect_url = $objClientCredit->getLinkUrl();

                                        $msg = "クレジットリンク型決済　遷移先URL：\n".$redirect_url;
                                        $objMdl->printLog($msg);
                                      //if( $event ) {
                                        if( !$objMdl->checkEccubeOldVersion011() ) {
                                            // 3.0.11以降
                                            $response = $this->app->redirect($redirect_url);
                                            $event->setResponse($response);
                                            return;
                                        } else {
                                            // 3.0.10以前
                                            header("Location: " . $redirect_url);
                                            exit;
                                        }
                                        break;
                                }
                            }
                        } else {
                            // コンビニ決済の場合
                            // 2017.1 拡張受注情報の作成
                            $objClientConvenience = new PAYDESIGN_PLG_DCHK_CONVENIENCE($this->app);
                            $PaydesignOrderInfo = $objClientConvenience->initPaydesignOrderInfo( $Order, $this->app['config']['PayDesign']['const']['PDP_ORDER_KIND_NORMAL'], '', $credit_uid_card_select_flg, 0 );
                            // 次ページに遷移
                            $redirect_url = $this->app->url('paydesign_shopping_convenience');
                          //if( $event ) {
                            if( !$objMdl->checkEccubeOldVersion011() ) {
                                // 3.0.11以降
                                $response = $this->app->redirect($redirect_url);
                                $event->setResponse($response);
                                return;
                            } else {
                                // 3.0.10以前
                                header("Location: " . $redirect_url);
                                exit;
                            }
                        }
                    }
                }

                // For Debug(Data Check)
                // if( count($form->get('credit_uid_card_select_flg')->getData()) > 0 ) {
                // print_r($form->get('credit_uid_card_select_flg')->getData()) ; }
                // exit;
            }
        } else {
            // 注文情報取得失敗時はエラー画面に遷移する
          //$this->app->addError('front.shopping.order.error');
          //if( $event ) {
            if( !$objMdl->checkEccubeOldVersion011() ) {
                // 3.0.11以降
                $response = $this->app->redirect($this->app->url('shopping_error'));
                $event->setResponse($response);
                return;
            } else {
                // 3.0.10以前
                return $this->app->redirect($this->app->url('shopping_error'));
            }
        }
    }

    // 【フロントページ】注文完了：Controller/Before
    // 定期マスタ作成処理
    public function onControllerShoppingCompleteBefore() {

        // 初期化・初期設定
        $app = $this->app;
        $objMdl =& PluginUtil::getInstance($this->app);
        $objMdl->install();

        // 注文データの取得
        $order_id = $this->app['session']->get('eccube.plugin.paydesign.orderId');
        if( !is_null($order_id) ) {
            $Order = $this->app['eccube.repository.order']->find($order_id); }

        // 定期マスタの作成
        if(!is_null($Order)) {
            $objClientPeriodic = new PAYDESIGN_PLG_DCHK_PERIODIC($this->app);
            $objClientPeriodic->createPeriodicInfo($Order);
        }
    }

    // 【管理画面】会員編集：Render/Before
    public function onRenderAdminCustomerEditBefore(FilterResponseEvent $event) {

        // 初期化・初期設定
        $app = $this->app;
        $customer_id = $this->app['request']->get('id');
        $objClientCredit = new PAYDESIGN_PLG_DCHK_CREDIT($this->app);

        // バージョン情報テーブルを取得
        $objMdl =& PluginUtil::getInstance($this->app);
        $objMdl->install();
        $oldVersion = $objMdl->getEccubeOldVersion();
        if( in_array(Constant::VERSION, $oldVersion) ) { $old_version = 1; }

        // 現在の通常注文用カード情報の取得
        $card_kind = $this->app['config']['PayDesign']['const']['PDP_CARD_KIND_NORMAL'];
        $NormalCardInfo = $objClientCredit->getCardInfoDataForDisp($customer_id, $card_kind);

        // 現在の定期注文用カード情報の取得
        $card_kind = $this->app['config']['PayDesign']['const']['PDP_CARD_KIND_PERIODIC'];
        $PeriodicCardInfo = $objClientCredit->getCardInfoDataForDisp($customer_id, $card_kind);

        // 追加ページのベース
        $crawler = new Crawler($event->getResponse()->getContent());
        // 追加ページデータ
        $appendTwig = 'PayDesign/View/admin/customer_edit.twig'; // view to include into the dom of Customer edit page
        $insert = $this->app['twig']->render($appendTwig, array(
            'old_version'                 => $old_version,
            'normal_card_info' => $NormalCardInfo,
            'periodic_card_info' => $PeriodicCardInfo,
        ));
        // $source = $crawler->html();
        // 文字化け対策
        $source = $this->getCrawlerHtml($crawler);
        try {
            // フォーム情報取得
            if( in_array(Constant::VERSION, $oldVersion) ) {
                // Before 3.0.4
                $form  = $crawler->filter('#customer_form .accordion')->last()->html();
            } else {
                // After 3.0.5
                $form  = $crawler->filter('div.col-md-9 > div.box')->last()->html();
            }
            $new_form = $form . $insert;
            $source = str_replace($form, $new_form, $source);
            // 追加ページ出力
            $event->getResponse()->setContent($source);
        } catch (\InvalidArgumentException $e) {
            // no-op
        }
    }

    // 【管理画面】会員削除：Controller/Before
    // 対象となる会員に紐づいた会員情報と定期マスタの削除処理
    public function onControllerAdminCustomerDeleteBefore() {

        // [memo]
        // 管理画面からの会員情報の削除の場合は、退会フラグのセットのみ

        // 会員IDの取得
        $customer_id = $this->app['request']->get('id');

        // カード情報と定期マスタの削除
        $this->deleteCardInfoAndPeriodicInfo($customer_id);
    }

    // 【管理画面】商品登録：Render/Before
    public function onRenderAdminProductNewBefore(FilterResponseEvent $event) {
        $app = $this->app;
        $request = $event->getRequest();
        $response = $event->getResponse();
        $product_id = $request->attributes->get('id');
        $response->setContent($this->getAdminProductHtml($request, $response, $product_id));
        $event->setResponse($response);
    }

    // 【管理画面】商品登録：Controller/After
    public function onControllerAdminProductNewAfter() {

        // プラグイン設定情報の取得
        $objMdl =& PluginUtil::getInstance($this->app);
        $objMdl->install();
        $objMdl->printLog("定期商品情報の登録開始");

        if ('POST' === $this->app['request']->getMethod()) {
            $objMdl->printLog("定期商品情報のセッション登録処理実行");

            // フォーム情報の取得
            $form = $this->app['form.factory']
                ->createBuilder('admin_product_periodic_info')
                ->getForm();
            $form->handleRequest($this->app['request']);

            // フォームデータの取得
            // $periodic_flg と $periodic_interval, $periodic_settle_method は初期値による設定値の調整
            $periodic_flg = 0 ;
            if( count($form->get('periodic_flg')->getData())>0 ) { $periodic_flg = 1 ; }
            $periodic_interval = 1 ;
            if( $form->get('periodic_interval')->getData() != "" ) {
                $periodic_interval = $form->get('periodic_interval')->getData() ; }
            $periodic_settle_method = 1 ;
            if( $form->get('periodic_settle_method')->getData() != "" ) {
                $periodic_settle_method = $form->get('periodic_settle_method')->getData() ; }
            $periodic_cutoff_day = $form->get('periodic_cutoff_day')->getData();
            $periodic_create_day = $form->get('periodic_create_day')->getData();
            $periodic_price = "";
            if( $form->get('periodic_price')->getData() != "" ) {
                $periodic_price = $form->get('periodic_price')->getData(); }

            if( $periodic_flg > 0 ) {
                // sessionのデータ保持
                $session = $this->app['request']->getSession();
                $session->set('eccube.admin.product_periodic_regist.regist_flg', 1);
                $session->set('eccube.admin.product_periodic_regist.periodic_flg', $periodic_flg);
                $session->set('eccube.admin.product_periodic_regist.periodic_interval', $periodic_interval);
                $session->set('eccube.admin.product_periodic_regist.periodic_settle_method', $periodic_settle_method);
                $session->set('eccube.admin.product_periodic_regist.periodic_cutoff_day', $periodic_cutoff_day);
                $session->set('eccube.admin.product_periodic_regist.periodic_create_day', $periodic_create_day);
                $session->set('eccube.admin.product_periodic_regist.periodic_price', $periodic_price);

                // 【登録用】定期商品情報をログに保存
                $msg  = "【登録用】定期商品情報\n";
                $msg .= "periodic_flg           = ".$periodic_flg."\n";
                $msg .= "periodic_interval      = ".$periodic_interval."\n";
                $msg .= "periodic_settle_method = ".$periodic_settle_method."\n";
                $msg .= "periodic_cutoff_day    = ".$periodic_cutoff_day."\n";
                $msg .= "periodic_create_day    = ".$periodic_create_day."\n";
                $msg .= "periodic_price         = ".$periodic_price;
                $objMdl->printLog($msg);
            }

        }

        $objMdl->printLog("定期商品情報の登録終了");
    }

    // 【管理画面】商品更新：Render/Before(onRenderAdminProductNewBefore() と同じ処理内容)
    public function onRenderAdminProductEditBefore(FilterResponseEvent $event) {
        $app = $this->app;
        $request = $event->getRequest();
        $response = $event->getResponse();
        $product_id = $request->attributes->get('id');
        $response->setContent($this->getAdminProductHtml($request, $response, $product_id));
        $event->setResponse($response);
    }

    // 【管理画面】商品登録・更新：Controller/After
    public function onControllerAdminProductEditAfter() {

        // プラグイン設定情報の取得
        $objMdl =& PluginUtil::getInstance($this->app);
        $objMdl->install();
        $objMdl->printLog("定期商品情報の登録・更新開始");

        // 商品IDの取得
        $id = $this->app['request']->attributes->get('id'); // product_id
        $Product = $this->app['eccube.repository.product']->find($id);

        // 定期購入機能が無効の場合は表示処理は行わない
        if(!$this->getHasPeriodic()) { $objMdl->printLog("定期商品情報の登録・更新：定期機能無効により終了"); return ; }

        $form = $this->app['form.factory']
            ->createBuilder('admin_product_periodic_info')
            ->getForm();

        // データ取得
        $ProductPeriodicInfo =
            $this->app['eccube.plugin.paydesign.repository.paydesign_product_periodic_info']->findOneBy( array('product_id'=>$id) );
        if( is_null($ProductPeriodicInfo) ) {
            $objMdl->printLog("定期商品情報が存在しなかったので作成");
            $ProductPeriodicInfo = new \Plugin\PayDesign\Entity\PayDesignProductPeriodicInfo() ;
        } else {
            // データが取得出来た場合、フォームにデータセット
            $objMdl->printLog("定期商品情報が存在しているので、フォームにデータを設定");
          //$form->get('periodic_flg')->setData($ProductPeriodicInfo->getPeriodicFlg());
            $form->get('periodic_price')->setData($ProductPeriodicInfo->getPeriodicPrice());
            $form->get('periodic_interval')->setData($ProductPeriodicInfo->getPeriodicInterval());
            $form->get('periodic_cutoff_day')->setData($ProductPeriodicInfo->getPeriodicCutoffDay());
            $form->get('periodic_create_day')->setData($ProductPeriodicInfo->getPeriodicCreateDay());
            $form->get('periodic_settle_method')->setData($ProductPeriodicInfo->getPeriodicSettleMethod());
        }

        $form->handleRequest($this->app['request']);

        if ('POST' === $this->app['request']->getMethod()) {
            $data = $form->getData();
            $err_flg = false;
            $periodic_price = $data['periodic_price'];
            $periodic_cutoff_day = $data['periodic_cutoff_day'];
            $periodic_create_day = $data['periodic_create_day'];
            if(!is_null($periodic_price) && !is_numeric($periodic_price)) {
                $this->app->clearMessage();
                $this->app->addError('※ 定期販売時価格は数字を設定してください。','admin');
                $err_flg = true;
            }
            if(!is_null($periodic_cutoff_day) && !is_null($periodic_create_day)) {
                if($periodic_cutoff_day == $periodic_create_day) {
                    $this->app->clearMessage();
                    $this->app->addError('※ 申込締切日と次回注文基準日に同じ日付を設定する事は出来ません ','admin');
                    $err_flg = true;
                }
                if($periodic_cutoff_day>27 && $periodic_create_day>$periodic_cutoff_day) {
                    $this->app->clearMessage();
                    $this->app->addError('※ 申込締切日が28日以降の場合、次回注文基準日に申込締切日より大きな日付を設定することは出来ません。','admin');
                    $err_flg = true;
                }
            }

            if( !is_null($Product) && !$err_flg ) {
                $objMdl->printLog("定期商品情報の更新処理開始");
                $periodic_has_class = $Product->hasProductClass();
                $ProductClasses = $Product->getProductClasses();
                if( $periodic_has_class ) {
                    // 規格あり・更新データの設定
                    foreach ($ProductClasses as $productClass) {
                        $product_class_id = $productClass->getId();
                        $this->updateProductPeriodicInfo($product_class_id, $id, $form, $periodic_has_class);
                    }
                } else {
                    // 規格なし・更新データの設定
                    $product_class_id = $ProductClasses[0]->getId();
                    $this->updateProductPeriodicInfo($product_class_id, $id, $form, $periodic_has_class);
                }
            }
        } else {
            if( !is_null($Product) ) {
                $objMdl->printLog("定期商品情報の登録処理開始");
                $session = $this->app['request']->getSession();
                $regist_flg = 0;
                if( !is_null($session->get('eccube.admin.product_periodic_regist.regist_flg')) ) {
                    $regist_flg = $session->get('eccube.admin.product_periodic_regist.regist_flg'); }
                if( $regist_flg > 0 ) {
                    // 商品データ登録用にパラメータ設定
                    $arrParam = array(
                        'periodic_flg' => $session->get('eccube.admin.product_periodic_regist.periodic_flg'),
                        'periodic_interval' => $session->get('eccube.admin.product_periodic_regist.periodic_interval'),
                        'periodic_settle_method' => $session->get('eccube.admin.product_periodic_regist.periodic_settle_method'),
                        'periodic_cutoff_day' => $session->get('eccube.admin.product_periodic_regist.periodic_cutoff_day'),
                        'periodic_create_day' => $session->get('eccube.admin.product_periodic_regist.periodic_create_day'),
                        'periodic_price' => $session->get('eccube.admin.product_periodic_regist.periodic_price'),
                    );

                    $periodic_has_class = $Product->hasProductClass();
                    $ProductClasses = $Product->getProductClasses();
                    if( $periodic_has_class ) {
                        // 規格あり・登録データの設定
                        foreach ($ProductClasses as $productClass) {
                            $product_class_id = $productClass->getId();
                            $this->registProductPeriodicInfo($product_class_id, $id, $arrParam);
                        }
                    } else {
                        // 規格なし・登録データの設定
                        $product_class_id = $ProductClasses[0]->getId();
                        $this->registProductPeriodicInfo($product_class_id, $id, $arrParam);
                    }

                    // 使用したセッションは消しておく
                    $session->remove('eccube.admin.product_periodic_regist.regist_flg');
                    $session->remove('eccube.admin.product_periodic_regist.periodic_flg');
                    $session->remove('eccube.admin.product_periodic_regist.periodic_interval');
                    $session->remove('eccube.admin.product_periodic_regist.periodic_settle_method');
                    $session->remove('eccube.admin.product_periodic_regist.periodic_cutoff_day');
                    $session->remove('eccube.admin.product_periodic_regist.periodic_create_day');
                    $session->remove('eccube.admin.product_periodic_regist.periodic_price');
                }
            }
        }

        $objMdl->printLog("定期商品情報の登録・更新終了");
    }

    // 【管理画面】商品削除：Controller/After
    public function onControllerAdminProductDeleteAfter() {

        // 商品IDの取得
        $id = $this->app['request']->attributes->get('id'); // product_id

        // 定期購入機能が無効の場合は表示処理は行わない
        if(!$this->getHasPeriodic()) { return ; }

        // 商品情報の取得
        $Product = $this->app['eccube.repository.product']->find($id);
        if( !is_null($Product) ) {
            if ($Product->hasProductClass()) {
                // 規格がある場合
                $ProductClasses = $Product->getProductClasses();
                foreach ($ProductClasses as $productClass) {
                    // 定期商品情報の取得
                    $product_class_id = $productClass->getId();
                    $ProductPeriodicInfo =
                        $this->app['eccube.plugin.paydesign.repository.paydesign_product_periodic_info']->find($product_class_id);
                    // 該当定期商品情報の論理削除
                    if( !is_null($ProductPeriodicInfo) ) {
                        $now = new \DateTime();
                        $ProductPeriodicInfo
                            ->setUpdateDate($now)
                            ->setDelFlg(1);
                        $this->app['orm.em']->persist($ProductPeriodicInfo);
                        $this->app['orm.em']->flush();
                    }    
                }
            } else {
                // 規格が無い場合（単品）
                $ProductPeriodicInfos =
                    $this->app['eccube.plugin.paydesign.repository.paydesign_product_periodic_info']->findBy(array('product_id'=>$id));
                if( !is_null($ProductPeriodicInfos) ) {
                    foreach( $ProductPeriodicInfos as $ProductPeriodicInfo ) { 
                        $now = new \DateTime();
                        $ProductPeriodicInfo
                            ->setUpdateDate($now)
                            ->setDelFlg(1);
                        $this->app['orm.em']->persist($ProductPeriodicInfo);
                        $this->app['orm.em']->flush();
                    }
                }
            }
        }
    }

    // 【管理画面】商品規格表示：Render/Before
    // src/Eccube/Controller/Admin/Product/ProductClassController.php の index の流用
    // $productClassForm の作成部分をカスタマイズ
    public function onRenderAdminProductClassBefore(FilterResponseEvent $event)
    {
        // 初期化
        $app = $this->app;
        $request = $event->getRequest();
        $response = $event->getResponse();
        $hasClassCategoryFlg = false;

        // 定期購入機能が無効の場合は表示処理は行わない
        // if(!$this->getHasPeriodic()) { return ; }
        return;

        // 商品情報の取得
        $product_id = $request->attributes->get('id');
        $Product = $app['eccube.repository.product']->find($product_id);
        if (!$Product) {
            // 商品情報取得エラー
            throw new NotFoundHttpException();
        }

        if (!$Product->hasProductClass()) {
            // 登録画面を表示

            $form = $app->form()
                ->add('class_name1', 'entity', array(
                    'class' => 'Eccube\Entity\ClassName',
                    'property' => 'name',
                    'empty_value' => '規格1を選択',
                    'constraints' => array(
                        new Assert\NotBlank(),
                    ),
                ))
                ->add('class_name2', 'entity', array(
                    'class' => 'Eccube\Entity\ClassName',
                    'property' => 'name',
                    'empty_value' => '規格2を選択',
                    'required' => false,
                ))
                ->getForm();

            $productClassForm = null;

            if ('POST' === $request->getMethod()) {

                $form->handleRequest($request);

                if ($form->isValid()) {

                    $data = $form->getData();

                    $ClassName1 = $data['class_name1'];
                    $ClassName2 = $data['class_name2'];

                    // 各規格が選択されている際に、分類を保有しているか確認
                    $class1Valied = $this->isValiedCategory($ClassName1);
                    $class2Valied = $this->isValiedCategory($ClassName2);

                    // 規格が選択されていないか、選択された状態で分類が保有されていれば、画面表示
                    if($class1Valied && $class2Valied){
                        $hasClassCategoryFlg = true;
                    }

                    if (!is_null($ClassName2) && $ClassName1->getId() == $ClassName2->getId()) {
                        // 規格1と規格2が同じ値はエラー

                        $form['class_name2']->addError(new FormError('規格1と規格2は、同じ値を使用できません。a'));

                    } else {

                        // 規格分類が設定されていない商品規格を取得
                        $orgProductClasses = $Product->getProductClasses();
                        $sourceProduct = $orgProductClasses[0];

                        // 規格分類が組み合わされた商品規格を取得
                        $ProductClasses = $this->createProductClasses($app, $Product, $ClassName1, $ClassName2);

                        // 組み合わされた商品規格にデフォルト値をセット
                        foreach ($ProductClasses as $productClass) {
                            $this->setDefualtProductClass($app, $productClass, $sourceProduct);
                        }

                        // 商品規格情報一覧フォームの作成
                        $productClassForm = $app->form()
                            ->add('product_classes', 'collection', array(
                                'type' => 'admin_product_class_custom',
                                'allow_add' => true,
                                'allow_delete' => true,
                                'data' => $ProductClasses,
                             ))
                            ->getForm()
                            ->createView();
                    }
                }
            }

            // 出力するページを作成
            $response_html = $app->renderView('PayDesign/View/admin/product_class.twig', array(
                'form' => $form->createView(),
                'classForm' => $productClassForm,
                'Product' => $Product,
                'not_product_class' => true,
                'error' => null,
                'has_class_category_flg' => $hasClassCategoryFlg,
            ));

        } else {

            // 既に商品規格が登録されている場合、商品規格画面を表示する

            // 既に登録されている商品規格を取得
            $ProductClasses = $this->getProductClassesExcludeNonClass($app, $Product);

            // 設定されている規格分類1、2を取得(商品規格の規格分類には必ず同じ値がセットされている)
            $ProductClass = $ProductClasses[0];
            $ClassName1 = $ProductClass->getClassCategory1()->getClassName();
            $ClassName2 = null;
            if (!is_null($ProductClass->getClassCategory2())) {
                $ClassName2 = $ProductClass->getClassCategory2()->getClassName();
            }

            // 規格分類が組み合わされた空の商品規格を取得
            $createProductClasses = $this->createProductClasses($app, $Product, $ClassName1, $ClassName2);

            $mergeProductClasses = array();

            // 商品税率が設定されている場合、商品税率を項目に設定
            $BaseInfo = $app['eccube.repository.base_info']->get();
            if ($BaseInfo->getOptionProductTaxRule() == Constant::ENABLED) {
                foreach ($ProductClasses as $class) {
                    if ($class->getTaxRule() && !$class->getTaxRule()->getDelFlg()) {
                        $class->setTaxRate($class->getTaxRule()->getTaxRate());
                    }
                }
            }

            // 登録済み商品規格と空の商品規格をマージ
            $flag = false;
            foreach ($createProductClasses as $createProductClass) {
                // 既に登録済みの商品規格にチェックボックスを設定
                foreach ($ProductClasses as $productClass) {
                    if ($productClass->getClassCategory1() == $createProductClass->getClassCategory1() &&
                            $productClass->getClassCategory2() == $createProductClass->getClassCategory2()) {
                                // チェックボックスを追加
                                $productClass->setAdd(true);
                                $flag = true;
                                break;
                    }
                }

                if (!$flag) {
                    $mergeProductClasses[] = $createProductClass;
                }

                $flag = false;
            }

            // 登録済み商品規格と空の商品規格をマージ
            foreach ($mergeProductClasses as $mergeProductClass) {
                // 空の商品規格にデフォルト値を設定
                $this->setDefualtProductClass($app, $mergeProductClass, $ProductClass);
                $ProductClasses->add($mergeProductClass);
            }

            // 商品規格情報一覧フォームの作成
            $productClassForm = $app->form()
                    ->add('product_classes', 'collection', array(
                        'type' => 'admin_product_class_custom',
                        'allow_add' => true,
                        'allow_delete' => true,
                        'data' => $ProductClasses,
                    ))
                    ->getForm()
                    ->createView();

            // 出力するページを作成
            $response_html = $app->renderView('PayDesign/View/admin/product_class.twig', array(
                'classForm' => $productClassForm,
                'Product' => $Product,
                'class_name1' => $ClassName1,
                'class_name2' => $ClassName2,
                'not_product_class' => false,
                'error' => null,
                'has_class_category_flg' => true,
            ));

        }

        // 作成したページをレスポンスで返す
        $response->setContent($response_html);
        $event->setResponse($response);
    }

    // 【管理画面】商品規格表示(編集)：Render/Before
    public function onRenderAdminProductClassEditBefore(FilterResponseEvent $event)
    {
        // 初期化
        $app = $this->app;
        $request = $event->getRequest();
        $response = $event->getResponse();
        $objMdl =& PluginUtil::getInstance($this->app);
        $objMdl->install();

        // 定期購入機能が無効の場合は表示処理は行わない
        if(!$this->getHasPeriodic()) { return ; }
        return;

        // 商品情報の取得
        $product_id = $request->attributes->get('id');
        $Product = $app['eccube.repository.product']->find($product_id);
        if (!$Product) {
            // 商品情報取得エラー
            throw new NotFoundHttpException();
        }

        $form = $app->form()
                ->add('product_classes', 'collection', array(
                    'type' => 'admin_product_class_custom',
                    'allow_add' => true,
                    'allow_delete' => true,
            ))
            ->getForm();

        // 既に登録されている商品規格を取得
        $ProductClasses = $this->getProductClassesExcludeNonClass($app, $Product);

        if ('POST' === $request->getMethod()) {
            $form->handleRequest($request);
            switch ($request->get('mode')) {
                case 'edit':
                    // 新規登録
                    if (count($ProductClasses) > 0) {
                        // 既に登録されていれば最初の画面に戻す
                        return $app->redirect($app->url('admin_product_product_class', array('id' => $product_id)));
                    }
                    $addProductClasses = array();
                    $tmpProductClass = null;
                    foreach ($form->get('product_classes') as $formData) {
                        // 追加対象の行をvalidate
                        $ProductClass = $formData->getData();
                        if ($ProductClass->getAdd()) {
                            if ($formData->isValid()) {
                                $addProductClasses[] = $ProductClass;
                            } else {
                                // 対象行のエラー
                                $response_html = $this->productClassRender($app, $Product, $ProductClass, true, $form);
                                $response->setContent($response_html);
                                $event->setResponse($response);
                                return;
                            }
                        }
                        $tmpProductClass = $ProductClass;
                    }
                    if (count($addProductClasses) == 0) {
                        // 対象がなければエラー
                        $error = array('message' => '商品規格が選択されていません。');
                        $response_html = $this->productClassRender($app, $Product, $tmpProductClass, true, $form, $error);
                        $response->setContent($response_html);
                        $event->setResponse($response);
                        return;
                    }
                    break;
                case 'update':
                    // 更新
                    if (count($ProductClasses) == 0) {
                        // 商品規格が0件であれば最初の画面に戻す
                        return $app->redirect($app->url('admin_product_product_class', array('id' => $product_id)));
                    }
                    $checkProductClasses = array();
                    $removeProductClasses = array();
                    $tempProductClass = null;
                    foreach ($form->get('product_classes') as $formData) {
                        // 追加対象の行をvalidate
                        $ProductClass = $formData->getData();
                        if ($ProductClass->getAdd()) {
                            if ($formData->isValid()) {
                                $checkProductClasses[] = $ProductClass;
                            } else {
                                $response_html = $this->productClassRender($app, $Product, $ProductClass, false, $form);
                                $response->setContent($response_html);
                                $event->setResponse($response);
                                return;
                            }
                        } else {
                            // 削除対象の行
                            $removeProductClasses[] = $ProductClass;
                        }
                        $tempProductClass = $ProductClass;
                    }
                    if (count($checkProductClasses) == 0) {
                        // 対象がなければエラー
                        $error = array('message' => '商品規格が選択されていません。');
                        $response_html = $this->productClassRender($app, $Product, $tempProductClass, false, $form, $error);
                        $response->setContent($response_html);
                        $event->setResponse($response);
                        return;
                    }
                    break;
                case 'delete':
                    // 削除
                    if (count($ProductClasses) == 0) {
                        // 既に商品が削除されていれば元の画面に戻す
                        return $app->redirect($app->url('admin_product_product_class', array('id' => $product_id)));
                    }
                    break;
                default:
                    break;
            }
            /*
            $ProductClass = $formData->getData();
            $response_html = $this->productClassRender($app, $Product, $ProductClass, true, $form);
            // 作成したページをレスポンスで返す
            $response->setContent($response_html);
            $event->setResponse($response);
            */
        }
    }

    // 【管理画面】商品規格編集：Controller/After
    // 定期販売価格の登録・更新・削除対応を行う。エラーチェック等は完了している前提
    public function onControllerAdminProductClassEditAfter() {
        /*
        $objMdl =& PluginUtil::getInstance($this->app);
        $objMdl->install();
        $objMdl->printLog("onControllerAdminProductClassEditAfter");
        */
    }

    /**
     * event for rendering the PaymentUtil layout before Order edit page
     * @param FilterResponseEvent $event
     */
    // 【管理画面】受注編集：Render/Before
    public function onRenderAdminOrderEditBefore(FilterResponseEvent $event)
    {
        $order_id = $this->app['request']->get('id');
        $Order = $this->app['orm.em']->getRepository('Eccube\Entity\Order')->find($order_id);

        // 2017.1.5 支払情報が取得出来ない場合は処理不可のため処理を終了
        if( is_null($Order->getPayment()) ) { return; }

        $PDOrder = $this->app['eccube.plugin.paydesign.repository.paydesign_order_info']->findOneBy( array( 'order_id' => $order_id, 'order_kind' => $this->app['config']['PayDesign']['const']['PDP_ORDER_KIND_NORMAL']));
        $PaymentMethod = $this->app['eccube.plugin.paydesign.repository.paydesign_payment_method']->findAll();
        $arrBuf = array() ;

        foreach ($PaymentMethod as $key => $value) {
            $arrBuf[$PaymentMethod[$key]->getId()] = $PaymentMethod[$key]->getMethod();
        }

        // 注文時の支払方法がペイデザイン決済かどうか
        $checkPayment = $this->app['eccube.plugin.paydesign.repository.paydesign_payment_method']->find($Order->getPayment()->getId());

        $source = $event->getResponse()->getContent();
        if (is_null($checkPayment))
        {
            // 注文時の支払方法がペイデザイン決済じゃない場合は、ペイデザイン以外を選択肢として作成。
            $arrPaymentId = array();
            foreach ($PaymentMethod as $key => $value) {
                array_push($arrPaymentId, $PaymentMethod[$key]->getId());
            }
            $arrPaymentRemove = '';
            foreach ($arrPaymentId as $key => $value) {
                $arrPaymentRemove .= '$("#order_Payment option[value='.$value.']").remove();
                        ';
            }

            $jqueryCode = '
                <script>
                    $(function(){
                        '.$arrPaymentRemove.'
                    });
                </script>
            ';

            $source .= $jqueryCode;
            $event->getResponse()->setContent($source);
            return;
        }

        // 個別別決済コードの取得
        $order_payment_code = $checkPayment->getMemo03(); // memo03 が個体別の決済コード
        // 受注拡張情報の取得
        $PaydesignOrderInfo =
            $this->app['eccube.plugin.paydesign.repository.paydesign_order_info']->findOneBy( array( 'order_id' => $Order->getId(), 'order_kind' => $this->app['config']['PayDesign']['const']['PDP_ORDER_KIND_NORMAL']));
        // 共通情報の取得
        if ($PaydesignOrderInfo) {
            $order_sid = $PaydesignOrderInfo->getSid();
            $payment_result_receive_date = $PaydesignOrderInfo->getResultReceiveDate();
        }
        if( is_null($payment_result_receive_date) ) {
            $payment_result_receive_date = '';
        } else {
            $payment_result_receive_date = $payment_result_receive_date->format('Y/m/d');
        }
        // 個別情報の取得
        switch($order_payment_code) {
            case 'credit':
                break;
            case 'convenience':
                break;
            default:
                break;
        }

        // 注文時の支払い方法がペイデザイン決済の場合、それのみを支払方法の選択肢とする。
        // ページに出力させる削除用のID一覧を作成
        $paymentObj = $this->app['orm.em']->getRepository('Eccube\Entity\Payment')->findAll(); // 全支払情報
        $arrPaymentId = array();
        foreach ($paymentObj as $key => $value) {
            array_push($arrPaymentId, $paymentObj[$key]->getId());
        }
        $arrPaymentRemove = array(); //''; 文字列初期化を配列型に変更（PHP7の場合、下記で配列使用なのに文字列初期化でエラーになる）
        foreach ($arrPaymentId as $key => $value) {
            if($Order->getPayment()->getId() != $value)
            {
                $arrPaymentRemove[] = $value;
            }
        }

        // バージョン情報テーブルを取得
        $objMdl =& PluginUtil::getInstance($this->app);
        $objMdl->install();
        $oldVersion = $objMdl->getEccubeOldVersion();
        if( in_array(Constant::VERSION, $oldVersion) ) { $old_version = 1; }

        // 追加ページのベース
        $crawler = new Crawler($event->getResponse()->getContent());
        // 追加ページデータ
        $appendTwig = 'PayDesign/View/admin/order_edit.twig'; // view to include into the dom of Order edit page
        $insert = $this->app['twig']->render($appendTwig, array(
            'old_version'                 => $old_version,
            'order_sid'                   => $order_sid,
            'order_status'                => $Order->getOrderStatus(),
            'order_payment_method'        => $Order->getPayment()->getMethod(),
            'order_payment_code'          => $order_payment_code,
            'payment_result_receive_date' => $payment_result_receive_date,
            'array_payment_remove'        => $arrPaymentRemove,
        ));
        // $source = $crawler->html();
        // 文字化け対策
        $source = $this->getCrawlerHtml($crawler);
        try {
            // フォーム情報取得
            if( in_array(Constant::VERSION, $oldVersion) ) {
                // Before 3.0.4
                $form  = $crawler->filter('#form1 .accordion')->last()->html();
            } else {
                // After 3.0.5
                $form  = $crawler->filter('div.col-md-12 > div.box')->last()->html();
            }
            $new_form = $form . $insert;
            $source = str_replace($form, $new_form, $source);
            // 追加ページ出力
            $event->getResponse()->setContent($source);
        } catch (\InvalidArgumentException $e) {
            // no-op
        }
    }

    // 【管理画面】受注登録：Render/Before
    // 受注登録時のお支払い方法にペイデザイン決済の決済方法を非表示にする。
    public function onRenderAdminOrderNewBefore(FilterResponseEvent $event)
    {
        $Payments = $this->app['eccube.plugin.paydesign.repository.paydesign_payment_method']->findAll();
        $arrPaymentId = array();

        foreach ($Payments as $key => $value) {
            array_push($arrPaymentId, $Payments[$key]->getId());
        }
        $source = $event->getResponse()->getContent();

        $arrPaymentRemove = '';
        foreach ($arrPaymentId as $key => $value) {
            $arrPaymentRemove .= '$("#order_Payment option[value='.$value.']").remove();
                    ';
        }

        $jqueryCode = '
            <script>
                $(function(){
                    '.$arrPaymentRemove.'
                });
            </script>
        ';

        $source .= $jqueryCode;
        $event->getResponse()->setContent($source);
    }

    // 【管理画面】受注編集：Controller/Before
    public function onControllerAdminOrderEditControllerBefore()
    {
        $request = $this->app['request'];

        // 現在の受注情報を取得
        $order_id = $request->get('id');
        $Order = $this->app['orm.em']->getRepository('Eccube\Entity\Order')->find($order_id);

        // sessionのデータ保持
        $session = $request->getSession();
        $session->set('eccube.admin.origin_order', $Order);
    }

    // 【管理画面】受注編集：Controller/After
    public function onControllerAdminOrderEditControllerAfter($event = NULL)
    {
        // プラグイン設定情報の取得
        $objMdl =& PluginUtil::getInstance($this->app);
        $objMdl->install();

        $request = $this->app['request'];
        $order_id = $request->get('id');
        $Order = $this->app['orm.em']->getRepository('Eccube\Entity\Order')->find($order_id);

        // 2017.1.5 支払情報が取得出来ない場合は処理不可のため処理を終了
        if( is_null($Order->getPayment()) ) { return; }

        $PaymentMethod = $this->app['eccube.plugin.paydesign.repository.paydesign_payment_method']->find($Order->getPayment()->getId());

        // データが取得できない決済（ペイデザイン決済以外）の場合は処理不要のため終了
        if( is_null($PaymentMethod) ) { return; }

        $payment_method_code = $PaymentMethod->getMemo03();

        // 変更前の受注情報を取得
        $session = $request->getSession();
        $OriginOrder = $session->get('eccube.admin.origin_order');
        // 取得後に破棄
        $session->remove('eccube.admin.origin_order');

        if ($request->getMethod() === 'POST' && $order_id) {
            $mode = $request->get('mode_type');
            if (!CommonUtil::isBlank($order_id) && CommonUtil::isInt($order_id)) {

                // 決済別に使用するモジュールを作成する
                switch($payment_method_code) {
                    case 'credit':
                        $objClient = new PAYDESIGN_PLG_DCHK_CREDIT($this->app);
                        break;
                    case 'convenience':
                        $objClient = new PAYDESIGN_PLG_DCHK_CONVENIENCE($this->app);
                        break ;
                }

                // ok_msg は $ret が true 且つ配列でない場合に設定可能
                $ret = false;
                $ok_msg = "" ;

                switch ($mode) {
                    case 'plg_paydesign_commit':          // [クレジット決済]売上確定実行
                        $ret = $objClient->commitOrder($Order);
                        if($ret && !is_array($ret)) { $ok_msg = "正常に売上確定処理が完了しました。"; }
                        break;
                    case 'plg_paydesign_auth_cancel':     // [クレジット決済]与信取消実行
                        $ret = $objClient->authCancelOrder($Order);
                        if($ret && !is_array($ret)) { $ok_msg = "正常に与信(オーソリ)取消処理が完了しました。"; }
                        break;
                    case 'plg_paydesign_cancel':          // [クレジット決済]取消(返品)実行
                        $ret = $objClient->cancelOrder($Order);
                        if($ret && !is_array($ret)) { $ok_msg = "正常に取消(返品)処理が完了しました。"; }
                        break;
                    case 'plg_paydesign_get_auth_status': // [クレジット決済]決済状態確認
                        $ret = $objClient->getAuthStatusOrder($Order);
                        if( !is_array($ret) ) {
                            if( $ret > 0 ) {
                                $retStatus = $this->app['config']['PayDesign']['const']['PDP_CREDIT_STATUS_'.$ret];
                                $ok_msg = $retStatus."状態である事を確認しました。";
                            }
                        }
                        break;
                    /******************************************************************************************
                    case 'plg_paydesign_get_status':      // [クレジット決済]決済状態(確定)確認
                        $ret = $objClient->getStatusOrder($Order);
                        if($ret && !is_array($ret)) { $ok_msg = "決済確定済みである事を確認しました。"; }
                        break;
                    ******************************************************************************************/
                    case 'plg_paydesign_conv_cancel':     // [コンビニ決済]申込キャンセル要求実行
                        $ret = $objClient->cancelOrder($Order);
                        if($ret && !is_array($ret)) { $ok_msg = "正常に申込キャンセル要求処理が完了しました。"; }
                        break;
                    default:
                        return;
                        break;
                }

                if (is_array($ret)) {
                    $error_message = 'エラーが発生しました。エラーコード【' . $ret['ErrCode'] . "】：" . $ret['ErrInfo'] ;
                    $this->app->addDanger($error_message, 'admin');
                } else if ($ret) {
                    $this->app->addSuccess($ok_msg, 'admin');
                } else {
                    $errors = $objClient->getError();
                    $error_message = '';
                    if (empty($errors)) {
                        $error_message .= '対象の変更は出来ない決済です。';
                    } else {
                        foreach ($errors as $errMess) {
                            $error_message .= $errMess;
                        }
                    }
                    $this->app->addDanger($error_message, 'admin');
                }

                $redirect_url = $this->app->url('admin_order_edit', array('id' => $order_id));
              //if( $event ) {
                if( !$objMdl->checkEccubeOldVersion011() ) {
                    // 3.0.11以降
                    $response = $this->app->redirect($redirect_url);
                    $event->setResponse($response);
                    return;
                } else {
                    // 3.0.10以前
                    header("Location: " . $redirect_url);
                    exit;
                }
            }
        }
    }

    // 【フロントページ】配送業者を取得
    private function findDeliveriesFromOrderDetails($app, $details) {

        $productTypes = array();
        foreach ($details as $detail) {
            $productTypes[] = $detail->getProductClass()->getProductType();
        }

        $qb = $app['orm.em']->createQueryBuilder();
        $deliveries = $qb->select("d")
            ->from("\Eccube\Entity\Delivery", "d")
            ->where($qb->expr()->in('d.ProductType', ':productTypes'))
            ->setParameter('productTypes', $productTypes)
            ->andWhere("d.del_flg = :delFlg")
            ->setParameter('delFlg', $app['config']['disabled'])
            ->orderBy("d.rank", "ASC")
            ->getQuery()
            ->getResult();

        return $deliveries;
    }

    // 【フロントページ】配送業者のフォームを設定
    private function setFormDelivery($form, $deliveries, $delivery = null) {

        // 配送業社の設定
        $form->add('delivery', 'entity', array(
            'class' => 'Eccube\Entity\Delivery',
            'property' => 'name',
            'choices' => $deliveries,
            'data' => $delivery,
        ));
    }

    // 【フロントページ】お届け日のフォームを設定
    private function setFormDeliveryDate($form, $Order, $app) {

        // お届け日の設定
        $minDate = 0;
        $deliveryDateFlag = false;

        // 配送時に最大となる商品日数を取得
        foreach ($Order->getOrderDetails() as $detail) {
            $deliveryDate = $detail->getProductClass()->getDeliveryDate();
            if (!is_null($deliveryDate)) {
                if ($minDate < $deliveryDate->getValue()) {
                    $minDate = $deliveryDate->getValue();
                }
                // 配送日数が設定されている
                $deliveryDateFlag = true;
            }
        }

        // 配達最大日数期間を設定
        $deliveryDates = array();

        // 配送日数が設定されている
        if ($deliveryDateFlag) {
            $period = new \DatePeriod (
                new \DateTime($minDate . ' day'),
                new \DateInterval('P1D'),
                new \DateTime($minDate + $app['config']['deliv_date_end_max'] . ' day')
            );

            foreach ($period as $day) {
                $deliveryDates[$day->format('Y/m/d')] = $day->format('Y/m/d');
            }
        }

        $form->add('deliveryDate', 'choice', array(
            'choices' => $deliveryDates,
            'required' => false,
            'empty_value' => '指定なし',
        ));
    }

    // 【フロントページ】お届け時間のフォームを設定
    private function setFormDeliveryTime($form, $delivery) {

        // お届け時間の設定
        $form->add('deliveryTime', 'entity', array(
            'class' => 'Eccube\Entity\DeliveryTime',
            'property' => 'deliveryTime',
            'choices' => $delivery->getDeliveryTimes(),
        ));
    }

    // 【フロントページ】支払方法のフォームを設定
    private function setFormPayment($form, $delivery, $payment = null) {

        // 支払い方法選択
        $paymentOptions = $delivery->getPaymentOptions();
        $payments = array();
        // 初期値で設定されている配送業社を設定
        foreach ($paymentOptions as $paymentOption) {
            $payments[] = $paymentOption->getPayment();
        }
        $form->add('payment', 'entity', array(
            'class' => 'Eccube\Entity\Payment',
            'property' => 'method',
            'choices' => $payments,
            'data' => $payment,
          //'expanded' => true,
        ));
    }

    // 【フロントページ】支払情報選択：カスタムフォームの追加処理
    // ['mapped' => false]オプションでシステムのValidate処理から除外している。
    private function setCreditUidCardSelectFlg($form) {
        $form->add('credit_uid_card_select_flg','choice', array(
            'choices' => array('1' => '　登録済みのカード情報を利用する'),
            'multiple' => true,
            'expanded' => true,
            'required' => true,
          //'mapped' => false,
            'data' => $this->CreditUidCardSelectFlg,
        ));
        // グローバル変数初期化
        $this->CreditUidCardSelectFlg = array(0);
    }

    // 【フロントページ】商品定期情報の登録
    private function registProductPeriodicInfo($product_class_id, $product_id, $arrParam ) {

        // 商品情報に紐づけられている商品定期情報を取得
        $ProductPeriodicInfo =
            $this->app['eccube.plugin.paydesign.repository.paydesign_product_periodic_info']->find($product_class_id);
        if( is_null($ProductPeriodicInfo) ) {
            $ProductPeriodicInfo = new \Plugin\PayDesign\Entity\PayDesignProductPeriodicInfo() ;
        } else {
            // 存在している場合は、処理不要（エラー）と判断して終了
            return;
        }

        // 登録
        $now = new \DateTime();
        $ProductPeriodicInfo
            ->setId($product_class_id)
            ->setProductId($product_id)
            ->setPeriodicFlg($arrParam['periodic_flg'])
            ->setPeriodicInterval($arrParam['periodic_interval'])
            ->setPeriodicCutoffDay($arrParam['periodic_cutoff_day'])
            ->setPeriodicCreateDay($arrParam['periodic_create_day'])
            ->setPeriodicSettleMethod($arrParam['periodic_settle_method'])
            ->setPeriodicPrice($arrParam['periodic_price'])
            ->setCreateDate($now)
            ->setUpdateDate($now)
            ->setDelFlg(0);

        // 登録処理実行
        $this->app['orm.em']->persist($ProductPeriodicInfo);
        $this->app['orm.em']->flush();
    }

    // 【フロントページ】商品定期情報の更新
    private function updateProductPeriodicInfo($product_class_id, $product_id, $form, $periodic_has_class ) {

        // 登録・更新パラメータの設定
        // $periodic_flg と $periodic_interval, $periodic_settle_method は初期値による設定値の調整
        $periodic_flg = 0 ;
        if( count($form->get('periodic_flg')->getData())>0 ) { $periodic_flg = 1 ; }
        $periodic_interval = 1 ;
        if( $form->get('periodic_interval')->getData() != "" ) {
            $periodic_interval = $form->get('periodic_interval')->getData() ; }
        $periodic_settle_method = 1 ;
        if( $form->get('periodic_settle_method')->getData() != "" ) {
            $periodic_settle_method = $form->get('periodic_settle_method')->getData() ; }

        // 商品情報に紐づけられている商品定期情報を取得
        $regist_flg = false ;
        $ProductPeriodicInfo =
            $this->app['eccube.plugin.paydesign.repository.paydesign_product_periodic_info']->find($product_class_id);
        if( is_null($ProductPeriodicInfo) ) {
            if( $periodic_flg > 0 ) {
                // 存在していない場合は、元を作成
                $regist_flg = true;
                $ProductPeriodicInfo = new \Plugin\PayDesign\Entity\PayDesignProductPeriodicInfo() ;
            } else {
                // 存在していなくて、定期設定がされていない場合は処理不要
                return;
            }
        }

        // 更新
        $now = new \DateTime();
        $ProductPeriodicInfo
            ->setId($product_class_id)
            ->setProductId($product_id)
            ->setPeriodicFlg($periodic_flg)
            ->setPeriodicInterval($periodic_interval)
            ->setPeriodicCutoffDay($form->get('periodic_cutoff_day')->getData())
            ->setPeriodicCreateDay($form->get('periodic_create_day')->getData())
            ->setPeriodicSettleMethod($periodic_settle_method)
            ->setPeriodicPrice($form->get('periodic_price')->getData())
            ->setUpdateDate($now)
            ->setDelFlg(0);

        // 新規登録の場合は登録日時も設定する
        if($regist_flg) { $ProductPeriodicInfo->setCreateDate($now); }

        // 更新処理実行
        $this->app['orm.em']->persist($ProductPeriodicInfo);
        $this->app['orm.em']->flush();
    }

    // 【フロントページ】HTML書き換え処理
    private function changeHtmlView(FilterResponseEvent $event, $class_name, $twig_filename, $append_param) {

        // 現在のソースを取得
        $source = $event->getResponse()->getContent();

        // DOMDocumentのエラー制御
        libxml_use_internal_errors(true);

        // DOMDocumentを用いてソースを取得する
        // loadHTML -> saveHTML での文字化け回避のため、mb_convert_encoding を行う
        $dom = new \DOMDocument();
      //$dom->loadHTML('<?xml encoding="UTF-8">' . $source);
        $dom->loadHTML(mb_convert_encoding('<?xml encoding="UTF-8">' . $source, 'HTML-ENTITIES', 'UTF-8'));
        $dom->encoding = "UTF-8";

        // ソースを結合したい部分を取得
        $domElements = $dom->getElementsByTagName("*");
        for ($i = 0; $i < $domElements->length; $i++) {
            if (@$domElements->item($i)->attributes->getNamedItem('class')->nodeValue == $class_name) {
                $elements[] = $domElements->item($i);
            }
        }

        // HTMLを変更
        if (!empty($elements)) {
            $template = $dom->createDocumentFragment();
            $template->appendXML($this->app['twig']->render($twig_filename, $append_param));
            $elements[0]->parentNode->replaceChild($template, $elements[0]);
            $html = html_entity_decode($dom->saveHTML(), ENT_NOQUOTES, 'UTF-8');
            $source = $event->getResponse()->setContent($html);
          //$source = $event->getResponse()->setContent($dom->saveHTML());
        }

        return $source;
    }

    // 【管理画面】定期商品情報レンダリング処理
    private function getAdminProductHtml($request, $response, $product_id = NULL) {

        // 定期購入機能の有効・無効フラグを取得する
        $has_periodic = 0 ;
        if($this->getHasPeriodic()) { $has_periodic = 1; }

        $periodic_has_class = false ;
        if($product_id) {
            // 商品情報を取得
            $Product = $this->app['eccube.repository.product']->find($product_id);
            $periodic_has_class = $Product->hasProductClass();
            // 商品情報に紐づけられている商品定期情報を取得
            if( $periodic_has_class ) {
                // 規格あり(1行あればOKなので、findOneBy)
                $ProductPeriodicInfo =
                    $this->app['eccube.plugin.paydesign.repository.paydesign_product_periodic_info']->findOneBy(array('product_id' => $product_id));
            } else {
                // 規格なし
                $ProductPeriodicInfo =
                    $this->app['eccube.plugin.paydesign.repository.paydesign_product_periodic_info']->findOneBy(array('product_id' => $product_id));
            }
        }
        $hasdata_flg = true;
        if( is_null($ProductPeriodicInfo) ) {
            $hasdata_flg = false;
            $ProductPeriodicInfo = new \Plugin\PayDesign\Entity\PayDesignProductPeriodicInfo() ;
            // フォームに表示させるデータの設定
            $ProductPeriodicInfo->setPeriodicInterval(1);
            $ProductPeriodicInfo->setPeriodicSettleMethod(1);
        }

        // 挿入するフォームのベースを作成
        $crawler = new Crawler($response->getContent());
        $form = $this->app['form.factory']
            ->createBuilder('admin_product_periodic_info')
            ->getForm();

        // フォームの設定
        if( $ProductPeriodicInfo->getPeriodicFlg() == 1 ) { $form['periodic_flg']->setData(array(1)); }
        $form->get('periodic_interval')->setData($ProductPeriodicInfo->getPeriodicInterval());
        $form->get('periodic_settle_method')->setData($ProductPeriodicInfo->getPeriodicSettleMethod());
        if($product_id && $hasdata_flg) {
            // データがNULLでない場合のみ設定
            if( !is_null($ProductPeriodicInfo->getPeriodicPrice())) {
                $form->get('periodic_price')->setData($ProductPeriodicInfo->getPeriodicPrice()); }
            if( !is_null($ProductPeriodicInfo->getPeriodicCutoffDay())) {
                $form->get('periodic_cutoff_day')->setData($ProductPeriodicInfo->getPeriodicCutoffDay()); }
            if( !is_null($ProductPeriodicInfo->getPeriodicCreateDay())) {
                $form->get('periodic_create_day')->setData($ProductPeriodicInfo->getPeriodicCreateDay()); }
        }

        // フォームの表示準備
        $form->handleRequest($request);
        $parts = $this->app->renderView( 'PayDesign/View/admin/product_periodic_info.twig', array(
            'form' => $form->createView(),
            'has_periodic' => $has_periodic,
            'periodic_has_class' => $periodic_has_class,
        ));

        // フォームの表示
        //$html = $crawler->html();
        // 文字化け対策
        $html = $this->getCrawlerHtml($crawler);
        try {
            $form  = $crawler->filter('#form1 .accordion')->last()->html();
            $new_form = $form . $parts;
            $html = str_replace($form, $new_form, $html);
        } catch (\InvalidArgumentException $e) {
            // no-op
        }
        return $html ;
    }

    // レンダリング時の文字化け対策用処理
    private function getCrawlerHtml($crawler) {
        $html = '';
        foreach ($crawler as $domElement) {
            $domElement->ownerDocument->formatOutput = true;
            $html .= $domElement->ownerDocument->saveHTML();
        }
        return html_entity_decode($html, ENT_NOQUOTES, 'UTF-8');
    }

    /*
     * 定期機能の使用判定
     */
    private function getHasPeriodic()
    {
        // プラグイン設定情報の取得
        $objMdl =& PluginUtil::getInstance($this->app);
        $objMdl->install();
        // 定期購入機能の有効・無効フラグを取得する
        $has_periodic = false ;
        if( $objMdl->getCreditPeriodicBuy() == 'true' ) { $has_periodic = true; }

        return $has_periodic;
    }

    /**
     * 規格1と規格2を組み合わせた商品規格を作成
     * ( src/Eccube/Controller/Admin/Product/ProductClassController.php の関数流用 )
     */
    private function createProductClasses($app, \Eccube\Entity\Product $Product, \Eccube\Entity\ClassName $ClassName1 = null, \Eccube\Entity\ClassName $ClassName2 = null)
    {
        $ClassCategories1 = array();
        if ($ClassName1) {
            $ClassCategories1 = $app['eccube.repository.class_category']->findBy(array('ClassName' => $ClassName1));
        }

        $ClassCategories2 = array();
        if ($ClassName2) {
            $ClassCategories2 = $app['eccube.repository.class_category']->findBy(array('ClassName' => $ClassName2));
        }

        $ProductClasses = array();
        foreach ($ClassCategories1 as $ClassCategory1) {
            if ($ClassCategories2) {
                foreach ($ClassCategories2 as $ClassCategory2) {
                    $ProductClass = $this->newProductClassCustom($app); // カスタム商品規格
                    $ProductClass->setProduct($Product);
                    $ProductClass->setClassCategory1($ClassCategory1);
                    $ProductClass->setClassCategory2($ClassCategory2);
                    $ProductClass->setTaxRate(null);
                    $ProductClass->setDelFlg(Constant::DISABLED);
                    $ProductClass->setPeriodicPrice(null);              // 【カスタム商品規格】定期商品価格
                    $ProductClasses[] = $ProductClass;
                }
            } else {
                $ProductClass = $this->newProductClassCustom($app);     // カスタム商品規格
                $ProductClass->setProduct($Product);
                $ProductClass->setClassCategory1($ClassCategory1);
                $ProductClass->setTaxRate(null);
                $ProductClass->setDelFlg(Constant::DISABLED);
                $ProductClass->setPeriodicPrice(null);                  // 【カスタム商品規格】定期商品価格
                $ProductClasses[] = $ProductClass;
            }

        }
        return $ProductClasses;
    }

    /**
     * 新しいカスタム商品規格を作成
     */
    private function newProductClassCustom(Application $app)
    {
        $ProductType = $app['eccube.repository.master.product_type']->find($app['config']['product_type_normal']);

        $PayDesignProductClassCustom = new \Plugin\PayDesign\Entity\PayDesignProductClassCustom() ;
        $PayDesignProductClassCustom->setProductType($ProductType);
        return $PayDesignProductClassCustom;
    }

    /**
     * 新しい商品規格を作成
     * ( src/Eccube/Controller/Admin/Product/ProductClassController.php の関数流用 )
     */
    private function newProductClass(Application $app)
    {
        $ProductType = $app['eccube.repository.master.product_type']->find($app['config']['product_type_normal']);

        $ProductClass = new ProductClass();
        $ProductClass->setProductType($ProductType);
        return $ProductClass;
    }

    /**
     * 規格なし商品を除いて商品規格を取得.
     * ( src/Eccube/Controller/Admin/Product/ProductClassController.php の関数流用 )
     *
     * @param Product $Product
     * @return \Eccube\Entity\ProductClass[]
     */
    private function getProductClassesExcludeNonClass(Application $app, \Eccube\Entity\Product $Product)
    {
        // 商品規格情報を取得してフィルタリング
        $ProductClasses = $Product->getProductClasses();
        $ProductClassesFilter = $ProductClasses->filter(function($ProductClass) {
            $ClassCategory1 = $ProductClass->getClassCategory1();
            $ClassCategory2 = $ProductClass->getClassCategory2();
            return ($ClassCategory1 || $ClassCategory2);
        });

        // 商品IDから定期販売価格一覧を取得
        $periodicPrice = array();
        $periodicPrice = $this->getProductClassPeriodicPriceList($app, $Product->getId());

        $ProductClassesCustom = array();
        foreach ($ProductClassesFilter as $ProductClass) {
            // カスタム商品規格情報を作成
            $ProductClassCustom = $this->newProductClassCustom($app);
            // カスタムに既存の商品規格情報をコピー
            $ProductClassCustom->setId($ProductClass->getId());
            $ProductClassCustom->setCode($ProductClass->getCode());
            $ProductClassCustom->setStock($ProductClass->getStock());
            $ProductClassCustom->setStockUnlimited($ProductClass->getStockUnlimited());
            $ProductClassCustom->setSaleLimit($ProductClass->getSaleLimit());
            $ProductClassCustom->setPrice01($ProductClass->getPrice01());
            $ProductClassCustom->setPrice02($ProductClass->getPrice02());
            $ProductClassCustom->setDeliveryFee($ProductClass->getDeliveryFee());
            $ProductClassCustom->setCreateDate($ProductClass->getCreateDate());
            $ProductClassCustom->setUpdateDate($ProductClass->getUpdateDate());
            $ProductClassCustom->setDelFlg($ProductClass->getDelFlg());
            $ProductClassCustom->setProduct($ProductClass->getProduct());
            $ProductClassCustom->setProductType($ProductClass->getProductType());
            $ProductClassCustom->setClassCategory1($ProductClass->getClassCategory1());
            $ProductClassCustom->setClassCategory2($ProductClass->getClassCategory2());
            $ProductClassCustom->setDeliveryDate($ProductClass->getDeliveryDate());
            $ProductClassCustom->setCreator($ProductClass->getCreator());
            $ProductClassCustom->setProductStock($ProductClass->getProductStock());
            $ProductClassCustom->setTaxRule($ProductClass->getTaxRule());

            // 定期販売価格を追加
            $ProductClassCustom->setPeriodicPrice($periodicPrice[$ProductClass->getId()]);

            // 一覧に保存
            $ProductClassesCustom[] = $ProductClassCustom;
        }

        return $ProductClassesCustom;
    }

    /**
     * デフォルトとなる商品規格を設定
     * ( src/Eccube/Controller/Admin/Product/ProductClassController.php の関数流用 )
     *
     * @param $productClassDest コピー先となる商品規格
     * @param $productClassOrig コピー元となる商品規格
     */
    private function setDefualtProductClass($app, $productClassDest, $productClassOrig) {
        $productClassDest->setDeliveryDate($productClassOrig->getDeliveryDate());
        $productClassDest->setProduct($productClassOrig->getProduct());
        $productClassDest->setProductType($productClassOrig->getProductType());
        $productClassDest->setCode($productClassOrig->getCode());
        $productClassDest->setStock($productClassOrig->getStock());
        $productClassDest->setStockUnlimited($productClassOrig->getStockUnlimited());
        $productClassDest->setSaleLimit($productClassOrig->getSaleLimit());
        $productClassDest->setPrice01($productClassOrig->getPrice01());
        $productClassDest->setPrice02($productClassOrig->getPrice02());
        $productClassDest->setDeliveryFee($productClassOrig->getDeliveryFee());

        // 個別消費税
        $BaseInfo = $app['eccube.repository.base_info']->get();
        if ($BaseInfo->getOptionProductTaxRule() == Constant::ENABLED) {
            if($productClassOrig->getTaxRate()) {
                $productClassDest->setTaxRate($productClassOrig->getTaxRate());
                if ($productClassDest->getTaxRule() && !$productClassDest->getTaxRule()->getDelFlg()) {
                    $productClassDest->getTaxRule()->setTaxRate($productClassOrig->getTaxRate());
                } else {
                    $taxrule = $app['eccube.repository.tax_rule']->newTaxRule();
                    $taxrule->setTaxRate($productClassOrig->getTaxRate());
                    $taxrule->setApplyDate(new \DateTime());
                    $taxrule->setProduct($productClassDest->getProduct());
                    $taxrule->setProductClass($productClassDest);
                    $productClassDest->setTaxRule($taxrule);
                }
            } else {
                if ($productClassDest->getTaxRule()) {
                    $productClassDest->getTaxRule()->setDelFlg(Constant::ENABLED);
                }
            }
        }
    }

    /**
     * 商品規格情報：登録、更新時のエラー画面表示
     *
     */
    private function productClassRender($app, $Product, $ProductClass, $not_product_class, $classForm, $error = null)
    {
        $ClassName1 = null;
        $ClassName2 = null;
        // 規格を取得
        if (isset($ProductClass)) {
            $ClassCategory1 = $ProductClass->getClassCategory1();
            if ($ClassCategory1) {
                $ClassName1 = $ClassCategory1->getClassName();
            }
            $ClassCategory2 = $ProductClass->getClassCategory2();
            if ($ClassCategory2) {
                $ClassName2 = $ClassCategory2->getClassName();
            }
        }

        $form = $app->form()
            ->add('class_name1', 'entity', array(
                'class' => 'Eccube\Entity\ClassName',
                'property' => 'name',
                'empty_value' => '規格1を選択',
                'data' => $ClassName1,
            ))
            ->add('class_name2', 'entity', array(
                'class' => 'Eccube\Entity\ClassName',
                'property' => 'name',
                'empty_value' => '規格2を選択',
                'data' => $ClassName2,
            ))
            ->getForm();

        return $app->renderView('PayDesign/View/admin/product_class.twig', array(
            'form' => $form->createView(),
            'classForm' => $classForm->createView(),
            'Product' => $Product,
            'class_name1' => $ClassName1,
            'class_name2' => $ClassName2,
            'not_product_class' => $not_product_class,
            'error' => $error,
            'has_class_category_flg' => true,
        ));
    }

    /**
     * 規格の分類判定( src/Eccube/Controller/Admin/Product/ProductClassController.php の関数流用 )
     *
     * @param Eccube\Entity\ClassName $ClassesName
     * @return boolean
     */
    private function isValiedCategory($class_name)
    {
        if (empty($class_name)) {
            return true;
        }
        if (count($class_name->getClassCategories()) < 1) {
            return false;
        }
        return true;
    }

    /*
     * 該当商品規格一覧の定期販売価格を取得
     * @param  integer $product_id
     * @return array   $periodicPrice 
     */
    private function getProductClassPeriodicPriceList(Application $app, $product_id)
    {
        $periodicPrice = array();

        // 定期販売価格を含む定期商品情報一覧を取得
        $ProductPeriodicInfo = $this->app['eccube.plugin.paydesign.repository.paydesign_product_periodic_info']->findBy(array('product_id'=>$product_id));

        if( !is_null($ProductPeriodicInfo) ) {
            foreach( $ProductPeriodicInfo as $val ) {
                $periodicPrice[$val->getId()] = $val->getPeriodicPrice();
            }
        }

        return $periodicPrice;
    }

    /*
     * 引数の会員情報に紐づいたカード情報と定期マスタの削除
     * @param  integer $customer_id
     * @return none
     */
    private function deleteCardInfoAndPeriodicInfo($customer_id)
    {
        if($customer_id) {
            // 定期マスタの削除
            $objClientPeriodic = new PAYDESIGN_PLG_DCHK_PERIODIC($this->app);
            $objClientPeriodic->deletePeriodicInfoAll($customer_id);

            // カード情報の削除
            $objClientCredit = new PAYDESIGN_PLG_DCHK_CREDIT($this->app);
            $objClientCredit->deleteCreditCardInfoAll($customer_id);
        }
    }
}
?>
